/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : GUI_USB_MSD_FS_Start.c
Purpose : Sample startup, using the file system driver as MSC storage driver
          showing the actual connection status on the display.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include "USB.h"
#include "USB_MSD.h"
#include "FS.h"
#include "BSP.h"
#include "GUI.h"

/*********************************************************************
*
*       Const data
*
**********************************************************************
*/

#ifndef GUI_CONST_STORAGE
  #define GUI_CONST_STORAGE const
#endif

static GUI_CONST_STORAGE unsigned char acusb_image[] = {
  /* RLE: 142 Pixels @ 000,000*/ 142, 0xFF, 0xFF,
  /* ABS: 016 Pixels @ 014,001*/ 0, 16, 0xFF, 0xFF, 0x5D, 0xEF, 0xFB, 0xE6, 0xBA, 0xDE, 0x59, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xC5, 0xB6, 0xC5, 0xF7, 0xCD, 0x38, 0xD6, 0x9A, 0xDE, 0xDB, 0xE6, 0x1C, 0xEF, 0x7D, 0xF7, 0xFF, 0xFF,
  /* RLE: 109 Pixels @ 030,001*/ 109, 0xFF, 0xFF,
  /* ABS: 004 Pixels @ 011,002*/ 0, 4, 0x3C, 0xEF, 0x18, 0xCE, 0xF3, 0xB4, 0xEF, 0x9B,
  /* RLE: 014 Pixels @ 015,002*/ 14, 0xCF, 0x93,
  /* ABS: 006 Pixels @ 029,002*/ 0, 6, 0xEF, 0x9B, 0xB2, 0xAC, 0x96, 0xC5, 0x79, 0xDE, 0x5D, 0xEF, 0xFF, 0xFF,
  /* RLE: 101 Pixels @ 035,002*/ 101, 0xFF, 0xFF,
  /* ABS: 003 Pixels @ 008,003*/ 0, 3, 0xDF, 0xFF, 0x59, 0xD6, 0x92, 0xAC,
  /* RLE: 006 Pixels @ 011,003*/ 6, 0xCF, 0x93,
  /* ABS: 009 Pixels @ 017,003*/ 0, 9, 0xCF, 0x9B, 0xEF, 0x9B, 0x10, 0x9C, 0x10, 0x9C, 0x10, 0x9C, 0xEF, 0x9B, 0xEF, 0x9B, 0xEF, 0x9B, 0xCF, 0x9B,
  /* RLE: 008 Pixels @ 026,003*/ 8, 0xCF, 0x93,
  /* ABS: 005 Pixels @ 034,003*/ 0, 5, 0xEF, 0x9B, 0xD3, 0xB4, 0xB6, 0xC5, 0x9A, 0xDE, 0x9E, 0xF7,
  /* RLE: 095 Pixels @ 039,003*/ 95, 0xFF, 0xFF,
  /* ABS: 027 Pixels @ 006,004*/ 0, 27, 0xFF, 0xFF, 0xD7, 0xCD, 0x10, 0x9C, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0x30, 0xA4, 0x92, 0xAC, 0x14, 0xB5, 0xB6, 0xBD, 0x38, 0xCE, 0x79, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6,
        0x59, 0xCE, 0x18, 0xCE, 0xF7, 0xC5, 0xB6, 0xC5, 0x34, 0xB5, 0xB2, 0xA4, 0x30, 0x9C, 0xEF, 0x93, 0xCF, 0x93, 0xCF, 0x93,
  /* RLE: 005 Pixels @ 033,004*/ 5, 0xCF, 0x93,
  /* ABS: 004 Pixels @ 038,004*/ 0, 4, 0xCF, 0x93, 0xB2, 0xAC, 0x18, 0xD6, 0x7D, 0xF7,
  /* RLE: 091 Pixels @ 042,004*/ 91, 0xFF, 0xFF,
  /* ABS: 033 Pixels @ 005,005*/ 0, 33, 0x9E, 0xF7, 0xD3, 0xB4, 0xCF, 0x93, 0xCF, 0x93, 0x10, 0x9C, 0x34, 0xB5, 0xBA, 0xD6, 0xFB, 0xE6, 0xFB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xD6, 0xBA, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6,
        0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x79, 0xD6, 0xF7, 0xC5, 0x76, 0xBD, 0xD3, 0xAC, 0x10, 0x9C, 0xEF, 0x93, 0xCF, 0x93, 0xCF, 0x93,
  /* RLE: 004 Pixels @ 038,005*/ 4, 0xCF, 0x93,
  /* ABS: 003 Pixels @ 042,005*/ 0, 3, 0x92, 0xAC, 0xF7, 0xCD, 0x3C, 0xEF,
  /* RLE: 087 Pixels @ 045,005*/ 87, 0xFF, 0xFF,
  /* ABS: 037 Pixels @ 004,006*/ 0, 37, 0x1C, 0xE7, 0x30, 0xA4, 0xCF, 0x93, 0xEF, 0x9B, 0xD7, 0xC5, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0xFB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xD6, 0xBA, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6,
        0x9A, 0xD6, 0x99, 0xD6, 0x78, 0xD6, 0x57, 0xDE, 0x78, 0xDE, 0x99, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x79, 0xD6, 0xF7, 0xC5, 0x55, 0xB5, 0x92, 0xA4, 0x10, 0x94, 0xEF, 0x93, 0xCF, 0x93,
  /* RLE: 004 Pixels @ 041,006*/ 4, 0xCF, 0x93,
  /* ABS: 004 Pixels @ 045,006*/ 0, 4, 0x51, 0xA4, 0xB6, 0xC5, 0x1C, 0xEF, 0xFF, 0xFF,
  /* RLE: 082 Pixels @ 049,006*/ 82, 0xFF, 0xFF,
  /* ABS: 041 Pixels @ 003,007*/ 0, 41, 0x7D, 0xF7, 0xEF, 0x9B, 0xCF, 0x93, 0x71, 0xA4, 0xFB, 0xE6, 0x7D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0xFB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0x36, 0xDE, 0x90, 0xDD, 0x0B, 0xDD, 0x87, 0xDC,
        0x02, 0xDC, 0xE0, 0xE3, 0xE0, 0xE3, 0xE0, 0xE3, 0x00, 0xE4, 0x01, 0xE4, 0x01, 0xE4, 0x22, 0xE4, 0x43, 0xE4, 0x85, 0xE4, 0x0A, 0xE5, 0x6E, 0xDD, 0xD2, 0xDD, 0x36, 0xDE, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x38, 0xCE, 0x96, 0xBD,
        0xF3, 0xAC, 0x30, 0x9C, 0xEF, 0x93, 0xCF, 0x93,
  /* RLE: 004 Pixels @ 044,007*/ 4, 0xCF, 0x93,
  /* ABS: 003 Pixels @ 048,007*/ 0, 3, 0x30, 0xA4, 0xB6, 0xC5, 0x7D, 0xF7,
  /* RLE: 079 Pixels @ 051,007*/ 79, 0xFF, 0xFF,
  /* ABS: 051 Pixels @ 002,008*/ 0, 51, 0xFF, 0xFF, 0xB2, 0xAC, 0xCF, 0x93, 0xF3, 0xB4, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xDE, 0x99, 0xDE, 0x8F, 0xDD, 0x65, 0xDC, 0xC0, 0xE3, 0xC0, 0xE3, 0xC0, 0xE3,
        0x22, 0xE4, 0xA6, 0xE4, 0xE8, 0xEC, 0x29, 0xED, 0x4A, 0xED, 0x6B, 0xED, 0x6B, 0xED, 0x29, 0xED, 0xE8, 0xEC, 0xC6, 0xE4, 0x85, 0xE4, 0x64, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x84, 0xE4, 0xC7, 0xE4, 0x2B, 0xDD, 0x8F, 0xDD, 0xF3, 0xD5,
        0x59, 0xD6, 0x59, 0xD6, 0x38, 0xCE, 0x96, 0xBD, 0xB2, 0xA4, 0x30, 0x9C, 0x30, 0x9C, 0xEF, 0x9B, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x9B, 0x34, 0xBD, 0x1C, 0xEF,
  /* RLE: 077 Pixels @ 053,008*/ 77, 0xFF, 0xFF,
  /* ABS: 054 Pixels @ 002,009*/ 0, 54, 0x9A, 0xDE, 0xCF, 0x93, 0x51, 0xA4, 0x9E, 0xF7, 0x9E, 0xF7, 0x7D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0x8F, 0xDD, 0xE1, 0xE3, 0xC0, 0xE3, 0x86, 0xE4, 0xCF, 0xED, 0xD6, 0xF6, 0x3A, 0xFF,
        0x3A, 0xFF, 0x19, 0xFF, 0xF8, 0xF6, 0xD7, 0xF6, 0xB6, 0xF6, 0x95, 0xF6, 0x74, 0xF6, 0x52, 0xF6, 0x32, 0xF6, 0x11, 0xF6, 0x10, 0xF6, 0xEF, 0xED, 0xCE, 0xED, 0x8C, 0xED, 0x28, 0xED, 0xC6, 0xEC, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4,
        0xE6, 0xE4, 0x6D, 0xDD, 0xF4, 0xD5, 0x38, 0xCE, 0x38, 0xCE, 0xF8, 0xCD, 0x55, 0xBD, 0xF3, 0xAC, 0xB3, 0xAC, 0x71, 0xA4, 0xEF, 0x9B, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xD3, 0xAC, 0xBA, 0xDE, 0xFF, 0xFF,
  /* RLE: 074 Pixels @ 056,009*/ 74, 0xFF, 0xFF,
  /* ABS: 056 Pixels @ 002,010*/ 0, 56, 0x92, 0xAC, 0xCF, 0x93, 0x9A, 0xDE, 0xBE, 0xF7, 0x9E, 0xF7, 0x7D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0xE9, 0xE4, 0xC0, 0xE3, 0xA6, 0xE4, 0xD7, 0xF6, 0xBE, 0xFF, 0x9D, 0xFF, 0x7B, 0xFF, 0x5A, 0xFF,
        0x39, 0xFF, 0x18, 0xF7, 0xD7, 0xF6, 0xB6, 0xF6, 0xB5, 0xF6, 0x74, 0xF6, 0x73, 0xF6, 0x52, 0xF6, 0x31, 0xF6, 0x10, 0xF6, 0xEF, 0xED, 0xCE, 0xED, 0xAD, 0xED, 0xAC, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x29, 0xED, 0xE7, 0xEC, 0xC5, 0xE4,
        0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0x07, 0xE5, 0x6E, 0xDD, 0xF4, 0xD5, 0x18, 0xCE, 0x18, 0xCE, 0xD7, 0xC5, 0x76, 0xBD, 0x55, 0xB5, 0xF3, 0xAC, 0x30, 0xA4, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0x51, 0xA4, 0x38, 0xD6, 0xDF, 0xFF,
  /* RLE: 071 Pixels @ 058,010*/ 71, 0xFF, 0xFF,
  /* ABS: 059 Pixels @ 001,011*/ 0, 59, 0x7D, 0xF7, 0xCF, 0x93, 0x71, 0xA4, 0xDF, 0xFF, 0xBE, 0xF7, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x76, 0xE6, 0xC0, 0xE3, 0x86, 0xE4, 0xDE, 0xFF, 0xDE, 0xFF, 0xBD, 0xFF, 0x7C, 0xFF, 0x5B, 0xFF,
        0x3A, 0xFF, 0x18, 0xFF, 0xF7, 0xF6, 0xD6, 0xF6, 0xB5, 0xF6, 0x94, 0xF6, 0x73, 0xF6, 0x52, 0xF6, 0x31, 0xF6, 0x10, 0xF6, 0xEF, 0xF5, 0xCF, 0xED, 0xCE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x6A, 0xED, 0x49, 0xED, 0x29, 0xED, 0x28, 0xED,
        0x07, 0xED, 0xE6, 0xEC, 0xE5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x28, 0xE5, 0x8F, 0xDD, 0xF7, 0xCD, 0x18, 0xCE, 0xF8, 0xCD, 0xD7, 0xC5, 0xB6, 0xC5, 0x75, 0xBD, 0x92, 0xAC, 0xCF, 0x9B, 0xCF, 0x93, 0xCF, 0x93, 0x10, 0x9C,
        0xB6, 0xC5, 0x9E, 0xF7,
  /* RLE: 069 Pixels @ 060,011*/ 69, 0xFF, 0xFF,
  /* ABS: 061 Pixels @ 001,012*/ 0, 61, 0x59, 0xD6, 0xCF, 0x93, 0xD7, 0xCD, 0xDF, 0xFF, 0x9E, 0xF7, 0x3C, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0xF2, 0xE5, 0xC0, 0xE3, 0x95, 0xF6, 0xDE, 0xFF, 0xBD, 0xFF, 0x9C, 0xFF, 0x7B, 0xFF, 0x5A, 0xFF,
        0x19, 0xFF, 0xF8, 0xF6, 0xD7, 0xF6, 0xB6, 0xF6, 0x95, 0xF6, 0x74, 0xF6, 0x53, 0xF6, 0x32, 0xF6, 0x11, 0xF6, 0x10, 0xF6, 0xEF, 0xED, 0xCE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x49, 0xED, 0x28, 0xED, 0x08, 0xED,
        0x07, 0xED, 0xE6, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x28, 0xED, 0x8F, 0xDD, 0xF6, 0xCD, 0xF8, 0xCD, 0xF7, 0xCD, 0xF7, 0xC5, 0xF8, 0xCD, 0xF8, 0xCD, 0x14, 0xB5, 0xEF, 0x9B, 0xCF, 0x93,
        0xCF, 0x93, 0xCF, 0x9B, 0x55, 0xBD, 0x3C, 0xEF,
  /* RLE: 067 Pixels @ 062,012*/ 67, 0xFF, 0xFF,
  /* ABS: 063 Pixels @ 001,013*/ 0, 63, 0x75, 0xC5, 0xCF, 0x93, 0xFB, 0xE6, 0xDF, 0xFF, 0x7D, 0xF7, 0x1C, 0xE7, 0x5D, 0xEF, 0x3C, 0xEF, 0x1C, 0xE7, 0x13, 0xE6, 0xC0, 0xE3, 0xF7, 0xF6, 0xDE, 0xFF, 0x9D, 0xFF, 0x7B, 0xFF, 0x5A, 0xFF, 0x39, 0xFF,
        0x18, 0xFF, 0xF7, 0xF6, 0xD6, 0xF6, 0x95, 0xF6, 0x94, 0xF6, 0x73, 0xF6, 0x52, 0xF6, 0x31, 0xF6, 0x10, 0xF6, 0xEF, 0xF5, 0xCE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x49, 0xED, 0x29, 0xED, 0x08, 0xED, 0x07, 0xED,
        0xE6, 0xEC, 0xE6, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x8E, 0xDD, 0xD5, 0xCD, 0xF7, 0xC5, 0xF7, 0xC5, 0xF7, 0xC5, 0x38, 0xCE, 0x7A, 0xD6, 0x96, 0xC5,
        0x51, 0xA4, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xF3, 0xB4, 0x3C, 0xEF,
  /* RLE: 065 Pixels @ 064,013*/ 65, 0xFF, 0xFF,
  /* ABS: 065 Pixels @ 001,014*/ 0, 65, 0xF3, 0xB4, 0xCF, 0x93, 0x9E, 0xF7, 0xDF, 0xFF, 0x7D, 0xEF, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0xDA, 0xE6, 0xE1, 0xE3, 0xF0, 0xF5, 0xBD, 0xFF, 0x9C, 0xFF, 0x5B, 0xFF, 0x3A, 0xFF, 0x18, 0xFF,
        0xF7, 0xF6, 0xD6, 0xF6, 0xB5, 0xF6, 0x94, 0xF6, 0x73, 0xF6, 0x52, 0xF6, 0x31, 0xF6, 0x10, 0xF6, 0xEF, 0xED, 0xCF, 0xED, 0xAE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x6A, 0xED, 0x4A, 0xED, 0x29, 0xED, 0x08, 0xED, 0x07, 0xED, 0xE7, 0xEC,
        0xE6, 0xEC, 0xC5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x68, 0xED, 0x8E, 0xDD, 0xD5, 0xCD, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0x59, 0xD6,
        0xFB, 0xE6, 0x59, 0xD6, 0xD3, 0xAC, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x9B, 0x75, 0xC5, 0x9E, 0xF7,
  /* RLE: 063 Pixels @ 066,014*/ 63, 0xFF, 0xFF,
  /* ABS: 067 Pixels @ 001,015*/ 0, 67, 0x92, 0xAC, 0xCF, 0x9B, 0xDF, 0xFF, 0xBE, 0xF7, 0x7D, 0xEF, 0x1C, 0xE7, 0xDB, 0xDE, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x0A, 0xDD, 0x22, 0xE4, 0x5A, 0xFF, 0x7B, 0xFF, 0x5A, 0xFF, 0x39, 0xFF, 0xF8, 0xF6,
        0xD7, 0xF6, 0xB6, 0xF6, 0x95, 0xF6, 0x74, 0xF6, 0x53, 0xF6, 0x32, 0xF6, 0x11, 0xF6, 0xF0, 0xF5, 0xEF, 0xED, 0xCE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x29, 0xED, 0x28, 0xED, 0x07, 0xED, 0xE7, 0xEC, 0xE6, 0xEC,
        0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0xB0, 0xDD, 0xD6, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5,
        0xD7, 0xC5, 0x79, 0xD6, 0x3D, 0xEF, 0xFB, 0xE6, 0xD3, 0xB4, 0xCF, 0x93, 0xCF, 0x93, 0x10, 0x9C, 0x18, 0xCE, 0xDF, 0xFF,
  /* RLE: 061 Pixels @ 068,015*/ 61, 0xFF, 0xFF,
  /* ABS: 068 Pixels @ 001,016*/ 0, 68, 0x51, 0xA4, 0x10, 0x9C, 0xDF, 0xFF, 0xBE, 0xF7, 0x7D, 0xEF, 0x1C, 0xE7, 0xBA, 0xDE, 0xFB, 0xE6, 0x1C, 0xE7, 0xFB, 0xE6, 0x98, 0xDE, 0x01, 0xE4, 0xC7, 0xE4, 0x3A, 0xFF, 0x39, 0xFF, 0x18, 0xFF, 0xF7, 0xF6,
        0xD6, 0xF6, 0xB5, 0xF6, 0x94, 0xF6, 0x73, 0xF6, 0x52, 0xF6, 0x31, 0xF6, 0x10, 0xF6, 0xEF, 0xED, 0xCE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x49, 0xED, 0x29, 0xED, 0x08, 0xED, 0x07, 0xED, 0xE6, 0xEC, 0xC5, 0xEC,
        0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAB, 0xED, 0xB1, 0xD5, 0xD6, 0xC5,
        0xB7, 0xC5, 0xB6, 0xC5, 0xB7, 0xC5, 0xBA, 0xDE, 0x9E, 0xF7, 0x79, 0xDE, 0x71, 0xA4, 0xCF, 0x93, 0xCF, 0x93, 0x51, 0xA4, 0x9A, 0xDE,
  /* RLE: 060 Pixels @ 069,016*/ 60, 0xFF, 0xFF,
  /* ABS: 070 Pixels @ 001,017*/ 0, 70, 0x30, 0xA4, 0x30, 0xA4, 0xDF, 0xFF, 0xBE, 0xF7, 0x7D, 0xEF, 0x1C, 0xE7, 0xBA, 0xDE, 0x9A, 0xD6, 0x1C, 0xE7, 0xFB, 0xDE, 0xFB, 0xDE, 0x90, 0xDD, 0xC0, 0xE3, 0xE8, 0xEC, 0x18, 0xFF, 0xF7, 0xF6, 0xD6, 0xF6,
        0xB5, 0xF6, 0x94, 0xF6, 0x73, 0xF6, 0x52, 0xF6, 0x31, 0xF6, 0x10, 0xF6, 0xEF, 0xF5, 0xCF, 0xED, 0xAE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x49, 0xED, 0xE6, 0xB3, 0x87, 0xD4, 0x07, 0xED, 0xE7, 0xEC, 0xE6, 0xE4, 0xC5, 0xEC,
        0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x47, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x69, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCC, 0xED,
        0xB3, 0xD5, 0xB6, 0xC5, 0xB6, 0xC5, 0xB6, 0xBD, 0xD7, 0xC5, 0xFB, 0xE6, 0x9E, 0xF7, 0xF7, 0xCD, 0x10, 0x9C, 0xCF, 0x93, 0xCF, 0x93, 0xD3, 0xB4, 0x5D, 0xEF,
  /* RLE: 058 Pixels @ 071,017*/ 58, 0xFF, 0xFF,
  /* ABS: 072 Pixels @ 001,018*/ 0, 72, 0x10, 0x9C, 0x30, 0xA4, 0xDF, 0xFF, 0xBE, 0xF7, 0x7D, 0xEF, 0x1C, 0xE7, 0xBA, 0xDE, 0x79, 0xD6, 0xBA, 0xDE, 0xFB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0x86, 0xDC, 0xC0, 0xE3, 0x2A, 0xED, 0xD6, 0xF6, 0xB6, 0xF6,
        0x95, 0xF6, 0x74, 0xF6, 0x53, 0xF6, 0x32, 0xF6, 0x11, 0xF6, 0xF0, 0xF5, 0xEF, 0xED, 0xCE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x29, 0xED, 0x25, 0x93, 0x00, 0x00, 0x21, 0x31, 0xA3, 0x82, 0x25, 0xCC, 0xC5, 0xEC,
        0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED,
        0xCB, 0xED, 0xCE, 0xE5, 0xB5, 0xC5, 0xB6, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x18, 0xCE, 0x3C, 0xEF, 0x5D, 0xEF, 0x75, 0xBD, 0xCF, 0x9B, 0xCF, 0x93, 0xEF, 0x9B, 0x38, 0xD6, 0xFF, 0xFF,
  /* RLE: 056 Pixels @ 073,018*/ 56, 0xFF, 0xFF,
  /* ABS: 073 Pixels @ 001,019*/ 0, 73, 0x51, 0xA4, 0xEF, 0x9B, 0xDF, 0xFF, 0x9E, 0xF7, 0x9E, 0xF7, 0x1C, 0xE7, 0xDA, 0xDE, 0x99, 0xD6, 0x38, 0xCE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0x36, 0xDE, 0xE0, 0xE3, 0xC0, 0xE3, 0x44, 0xE4, 0xF0, 0xF5,
        0x74, 0xF6, 0x73, 0xF6, 0x52, 0xF6, 0x31, 0xF6, 0x10, 0xF6, 0xEF, 0xED, 0xCE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x29, 0xED, 0x28, 0xED, 0xC7, 0xDC, 0x20, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE1, 0x28,
        0x23, 0x93, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED,
        0xCB, 0xED, 0xEB, 0xF5, 0xEC, 0xED, 0xD1, 0xD5, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x59, 0xD6, 0x7D, 0xEF, 0xFB, 0xE6, 0xD3, 0xAC, 0xCF, 0x93, 0xCF, 0x93, 0xD3, 0xB4, 0x7D, 0xF7,
  /* RLE: 055 Pixels @ 074,019*/ 55, 0xFF, 0xFF,
  /* ABS: 074 Pixels @ 001,020*/ 0, 74, 0xB2, 0xAC, 0xCF, 0x93, 0x9E, 0xF7, 0x9E, 0xF7, 0x9E, 0xF7, 0x1C, 0xE7, 0xDB, 0xDE, 0x9A, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0x90, 0xDD, 0xC0, 0xE3, 0xE0, 0xE3, 0xE0, 0xE3,
        0xC7, 0xEC, 0xCF, 0xED, 0x31, 0xF6, 0x10, 0xF6, 0xEF, 0xF5, 0xCF, 0xED, 0xAE, 0xED, 0xAD, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x49, 0xED, 0x29, 0xED, 0x08, 0xED, 0x07, 0xED, 0xC2, 0x51, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x41, 0x39,
        0xC6, 0xE4, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x06, 0xB4, 0xA4, 0x72, 0x08, 0xDD, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED,
        0xEB, 0xED, 0xEB, 0xF5, 0xEC, 0xF5, 0x0C, 0xF6, 0xEE, 0xE5, 0xB4, 0xC5, 0x96, 0xBD, 0x96, 0xBD, 0x76, 0xBD, 0x96, 0xBD, 0xDB, 0xDE, 0x9E, 0xF7, 0xF7, 0xCD, 0xEF, 0x9B, 0xCF, 0x93, 0x10, 0x9C, 0x79, 0xDE,
  /* RLE: 054 Pixels @ 075,020*/ 54, 0xFF, 0xFF,
  /* ABS: 076 Pixels @ 001,021*/ 0, 76, 0xF3, 0xB4, 0xCF, 0x93, 0x3C, 0xEF, 0x9E, 0xF7, 0x9E, 0xF7, 0x3C, 0xE7, 0xDB, 0xDE, 0x9A, 0xD6, 0x59, 0xD6, 0x18, 0xCE, 0x59, 0xCE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xD6, 0xC9, 0xDC, 0xE0, 0xE3, 0xE0, 0xE3,
        0x00, 0xE4, 0x01, 0xE4, 0x42, 0xE4, 0xC6, 0xE4, 0x4A, 0xED, 0x8C, 0xED, 0x8C, 0xED, 0x8C, 0xED, 0x6B, 0xED, 0x4A, 0xED, 0x4A, 0xED, 0x29, 0xED, 0x08, 0xED, 0x07, 0xED, 0xE6, 0xEC, 0xC4, 0xB3, 0xA3, 0x82, 0x24, 0xCC, 0xA1, 0x51, 0x00, 0x00,
        0x41, 0x39, 0x85, 0xD4, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x44, 0x93, 0xC1, 0x20, 0x00, 0x00, 0x00, 0x00, 0xA1, 0x18, 0x47, 0xBC, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xCB, 0xED,
        0xEB, 0xED, 0xEB, 0xF5, 0xEC, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0D, 0xEE, 0xD1, 0xDD, 0x95, 0xBD, 0x76, 0xBD, 0x75, 0xBD, 0x75, 0xBD, 0x38, 0xCE, 0x7D, 0xEF, 0xFB, 0xE6, 0x92, 0xAC, 0xCF, 0x93, 0xCF, 0x93, 0x34, 0xBD, 0xBE, 0xF7,
  /* RLE: 052 Pixels @ 077,021*/ 52, 0xFF, 0xFF,
  /* ABS: 077 Pixels @ 001,022*/ 0, 77, 0x75, 0xC5, 0xCF, 0x93, 0xBA, 0xDE, 0x9E, 0xF7, 0x7D, 0xEF, 0x5D, 0xEF, 0xDB, 0xDE, 0x9A, 0xD6, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xC5, 0x9A, 0xD6, 0xBA, 0xDE, 0xBA, 0xD6, 0x99, 0xD6, 0x44, 0xDC, 0xE0, 0xE3,
        0x01, 0xE4, 0x01, 0xE4, 0x01, 0xE4, 0x21, 0xE4, 0x22, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x63, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA5, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0xE3, 0x8A,
        0x20, 0x08, 0x60, 0x10, 0xC5, 0xAB, 0x07, 0xED, 0x27, 0xED, 0xC7, 0xDC, 0x82, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x05, 0x83, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xEB, 0xED,
        0xEB, 0xF5, 0xEB, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x2E, 0xEE, 0xB4, 0xC5, 0x75, 0xBD, 0x75, 0xBD, 0x75, 0xBD, 0x96, 0xBD, 0xFC, 0xE6, 0x7D, 0xF7, 0x96, 0xC5, 0xCF, 0x93, 0xCF, 0x93, 0x30, 0xA4, 0xBA, 0xE6,
  /* RLE: 051 Pixels @ 078,022*/ 51, 0xFF, 0xFF,
  /* ABS: 079 Pixels @ 001,023*/ 0, 79, 0x18, 0xD6, 0xCF, 0x93, 0x18, 0xCE, 0x9E, 0xF7, 0x7D, 0xEF, 0x5D, 0xEF, 0xFB, 0xDE, 0xBA, 0xDE, 0x79, 0xD6, 0x38, 0xCE, 0xF7, 0xC5, 0xD7, 0xC5, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0x36, 0xD6, 0x22, 0xE4,
        0x01, 0xE4, 0x01, 0xE4, 0x21, 0xE4, 0x21, 0xE4, 0x22, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x62, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC,
        0x05, 0xBC, 0xC1, 0x20, 0x00, 0x00, 0x83, 0x72, 0x07, 0xED, 0x27, 0xED, 0x48, 0xED, 0x04, 0x83, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA6, 0x9B, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xCB, 0xED, 0xEB, 0xED,
        0xEB, 0xF5, 0xEC, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0xF1, 0xDD, 0x75, 0xBD, 0x75, 0xBD, 0x55, 0xBD, 0x55, 0xB5, 0x59, 0xD6, 0x9E, 0xF7, 0xBA, 0xDE, 0x51, 0xA4, 0xCF, 0x93, 0xCF, 0x93,
        0xB6, 0xC5, 0xFF, 0xFF,
  /* RLE: 049 Pixels @ 080,023*/ 49, 0xFF, 0xFF,
  /* ABS: 080 Pixels @ 001,024*/ 0, 80, 0xDB, 0xE6, 0xCF, 0x93, 0x75, 0xBD, 0x9E, 0xF7, 0x7D, 0xEF, 0x5D, 0xEF, 0x1C, 0xE7, 0xBA, 0xDE, 0x79, 0xD6, 0x38, 0xCE, 0x17, 0xCE, 0xD6, 0xC5, 0xF7, 0xC5, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0xF3, 0xDD,
        0x01, 0xE4, 0x01, 0xE4, 0x21, 0xE4, 0x22, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xC5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC,
        0x06, 0xED, 0xC6, 0xDC, 0xE2, 0x51, 0x00, 0x00, 0x41, 0x39, 0xA7, 0xD4, 0x48, 0xED, 0x48, 0xED, 0x46, 0xBC, 0xA2, 0x49, 0x22, 0x31, 0x00, 0x00, 0xE3, 0x51, 0x49, 0xDD, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xCB, 0xED, 0xEB, 0xF5,
        0xEB, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x4F, 0xEE, 0xB3, 0xCD, 0x55, 0xBD, 0x55, 0xB5, 0x55, 0xB5, 0xB6, 0xC5, 0x3C, 0xE7, 0x7D, 0xEF, 0x55, 0xBD, 0xCF, 0x93,
        0xCF, 0x93, 0xD3, 0xB4, 0x9E, 0xF7,
  /* RLE: 048 Pixels @ 081,024*/ 48, 0xFF, 0xFF,
  /* ABS: 081 Pixels @ 001,025*/ 0, 81, 0x7D, 0xF7, 0xCF, 0x93, 0xD3, 0xB4, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0xDA, 0xDE, 0x9A, 0xD6, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xC5, 0x96, 0xBD, 0x18, 0xCE, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6,
        0xB1, 0xDD, 0x21, 0xE4, 0x21, 0xE4, 0x22, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA5, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC,
        0x06, 0xED, 0x06, 0xED, 0x27, 0xED, 0x24, 0x93, 0x20, 0x08, 0x60, 0x10, 0xC5, 0xAB, 0x48, 0xED, 0x68, 0xED, 0x69, 0xED, 0x69, 0xED, 0xA4, 0x72, 0x00, 0x00, 0xC1, 0x20, 0x88, 0xBC, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xEB, 0xED, 0xEB, 0xF5,
        0xEC, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8E, 0xF6, 0x11, 0xDE, 0x74, 0xBD, 0x55, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x9A, 0xD6, 0x9E, 0xF7, 0x38, 0xD6,
        0xEF, 0x9B, 0xCF, 0x93, 0x30, 0xA4, 0x1C, 0xEF,
  /* RLE: 047 Pixels @ 082,025*/ 47, 0xFF, 0xFF,
  /* ABS: 082 Pixels @ 001,026*/ 0, 82, 0xFF, 0xFF, 0xEF, 0x9B, 0x51, 0xA4, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0xFB, 0xE6, 0x9A, 0xD6, 0x79, 0xD6, 0x38, 0xCE, 0xF7, 0xC5, 0xB6, 0xC5, 0x75, 0xBD, 0x38, 0xCE, 0xBA, 0xD6, 0xBA, 0xD6,
        0xBA, 0xD6, 0x6D, 0xDD, 0x21, 0xE4, 0x22, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0x06, 0xED,
        0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x66, 0xC4, 0xE1, 0x28, 0x00, 0x00, 0x84, 0x72, 0x48, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x07, 0xAC, 0x60, 0x10, 0x20, 0x00, 0x46, 0x8B, 0xCB, 0xED, 0xCB, 0xED, 0xEB, 0xED, 0xEB, 0xF5,
        0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0x70, 0xEE, 0xD3, 0xCD, 0x55, 0xB5, 0x35, 0xB5, 0x34, 0xB5, 0xF8, 0xCD, 0x7D, 0xEF,
        0xFB, 0xE6, 0x51, 0xA4, 0xCF, 0x93, 0xEF, 0x9B, 0x38, 0xD6,
  /* RLE: 047 Pixels @ 083,026*/ 47, 0xFF, 0xFF,
  /* ABS: 083 Pixels @ 002,027*/ 0, 83, 0x92, 0xAC, 0xCF, 0x93, 0x5D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0xBA, 0xDE, 0x79, 0xD6, 0x59, 0xCE, 0x18, 0xCE, 0xD7, 0xC5, 0x96, 0xBD, 0x75, 0xBD, 0x59, 0xCE, 0xBA, 0xD6, 0xBA, 0xDE,
        0xBA, 0xDE, 0x0A, 0xE5, 0x42, 0xE4, 0x42, 0xE4, 0x42, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x83, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA5, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED,
        0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x47, 0xED, 0x07, 0xE5, 0x03, 0x5A, 0x00, 0x00, 0x42, 0x39, 0xE8, 0xD4, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0x09, 0xD5, 0x62, 0x39, 0x00, 0x00, 0xC3, 0x49, 0x8A, 0xE5, 0xEB, 0xF5, 0xEB, 0xF5, 0x0C, 0xF6,
        0x0C, 0xF6, 0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xAF, 0xF6, 0x32, 0xDE, 0x54, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x96, 0xBD, 0x3C, 0xEF,
        0x5D, 0xEF, 0xF3, 0xB4, 0xCF, 0x93, 0xCF, 0x93, 0x55, 0xBD, 0xDF, 0xFF,
  /* RLE: 045 Pixels @ 085,027*/ 45, 0xFF, 0xFF,
  /* ABS: 084 Pixels @ 002,028*/ 0, 84, 0x34, 0xBD, 0xCF, 0x93, 0xBA, 0xDE, 0x7D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x3C, 0xE7, 0xFB, 0xE6, 0x9A, 0xD6, 0x59, 0xD6, 0x38, 0xCE, 0xF7, 0xC5, 0xB6, 0xC5, 0x75, 0xBD, 0x55, 0xBD, 0x79, 0xD6, 0xBA, 0xDE,
        0xBA, 0xDE, 0xBA, 0xDE, 0xE8, 0xE4, 0x42, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x02, 0x62,
        0xE1, 0x28, 0x21, 0x31, 0x24, 0x93, 0x07, 0xE5, 0x48, 0xED, 0x68, 0xED, 0x85, 0x9B, 0x40, 0x08, 0x60, 0x10, 0xE6, 0xAB, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xA5, 0x6A, 0x00, 0x00, 0x42, 0x31, 0xAB, 0xE5, 0xEC, 0xF5, 0x0C, 0xF6,
        0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0x91, 0xEE, 0x74, 0xBD, 0x34, 0xB5, 0x34, 0xB5, 0x55, 0xB5,
        0xDB, 0xDE, 0x9E, 0xF7, 0xD7, 0xCD, 0xCF, 0x93, 0xCF, 0x93, 0x92, 0xAC, 0x7D, 0xF7,
  /* RLE: 044 Pixels @ 086,028*/ 44, 0xFF, 0xFF,
  /* ABS: 085 Pixels @ 002,029*/ 0, 85, 0xF7, 0xCD, 0xCF, 0x93, 0x38, 0xCE, 0x7D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0xDB, 0xDE, 0x79, 0xD6, 0x59, 0xCE, 0x18, 0xCE, 0xF7, 0xC5, 0xB6, 0xBD, 0x75, 0xBD, 0x75, 0xBD, 0x9A, 0xD6,
        0xBA, 0xDE, 0xDB, 0xDE, 0xB9, 0xDE, 0xA6, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x63, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xC5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0xA2, 0x49,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC1, 0x20, 0xE7, 0xD4, 0x68, 0xED, 0x68, 0xED, 0xA7, 0xCC, 0x01, 0x29, 0x00, 0x00, 0x84, 0x6A, 0x8A, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0x24, 0x5A, 0x00, 0x00, 0xC3, 0x41, 0x0C, 0xF6, 0x0C, 0xF6,
        0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xD1, 0xF6, 0xD3, 0xCD, 0x34, 0xB5, 0x14, 0xB5,
        0x14, 0xB5, 0x59, 0xD6, 0x9E, 0xF7, 0x9A, 0xDE, 0x10, 0x9C, 0xCF, 0x93, 0x30, 0xA4, 0x1C, 0xEF,
  /* RLE: 043 Pixels @ 087,029*/ 43, 0xFF, 0xFF,
  /* ABS: 086 Pixels @ 002,030*/ 0, 86, 0x9A, 0xDE, 0xCF, 0x93, 0x96, 0xC5, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0x9A, 0xD6, 0x79, 0xD6, 0x38, 0xCE, 0x17, 0xCE, 0xD7, 0xC5, 0x96, 0xBD, 0x55, 0xBD, 0x75, 0xBD,
        0xBA, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0x98, 0xE6, 0x85, 0xE4, 0x63, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0xC6, 0xDC,
        0x62, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x72, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x69, 0xE5, 0x44, 0x62, 0x00, 0x00, 0x42, 0x31, 0x09, 0xD5, 0xCA, 0xED, 0xCB, 0xED, 0xE9, 0xCC, 0x00, 0x00, 0x00, 0x00, 0x0A, 0xCD, 0x0C, 0xF6,
        0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x52, 0xDE, 0x34, 0xB5,
        0x14, 0xB5, 0x14, 0xAD, 0xD7, 0xC5, 0x7D, 0xEF, 0x3C, 0xEF, 0x92, 0xAC, 0xCF, 0x93, 0xEF, 0x9B, 0x9A, 0xDE,
  /* RLE: 042 Pixels @ 088,030*/ 42, 0xFF, 0xFF,
  /* ABS: 087 Pixels @ 002,031*/ 0, 87, 0x3C, 0xEF, 0xCF, 0x93, 0xD3, 0xB4, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0xFB, 0xDE, 0x9A, 0xD6, 0x59, 0xD6, 0x38, 0xCE, 0xF7, 0xC5, 0xB6, 0xC5, 0x95, 0xBD, 0x55, 0xBD,
        0x96, 0xBD, 0xDB, 0xDE, 0xDB, 0xDE, 0xFB, 0xE6, 0x56, 0xE6, 0x84, 0xE4, 0x83, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xC5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED,
        0x27, 0xED, 0x46, 0xC4, 0x04, 0x83, 0x80, 0x18, 0x20, 0x00, 0x05, 0x83, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x8A, 0xED, 0xC6, 0xA3, 0x40, 0x08, 0x60, 0x10, 0x07, 0xA4, 0xCB, 0xED, 0x8A, 0xE5, 0x00, 0x00, 0x00, 0x00, 0xE6, 0x72, 0x0C, 0xF6,
        0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xB2, 0xEE,
        0x74, 0xBD, 0x14, 0xAD, 0x14, 0xAD, 0x75, 0xB5, 0x3C, 0xE7, 0x7D, 0xEF, 0x14, 0xB5, 0xCF, 0x93, 0xCF, 0x93, 0x18, 0xCE,
  /* RLE: 041 Pixels @ 089,031*/ 41, 0xFF, 0xFF,
  /* ABS: 089 Pixels @ 002,032*/ 0, 89, 0xFF, 0xFF, 0x10, 0x9C, 0x10, 0x9C, 0x5D, 0xEF, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xDB, 0xDE, 0x79, 0xD6, 0x59, 0xCE, 0x18, 0xCE, 0xF7, 0xC5, 0xB6, 0xC5, 0x95, 0xBD,
        0x54, 0xBD, 0xB6, 0xC5, 0xDB, 0xDE, 0xFB, 0xE6, 0xDB, 0xDE, 0x34, 0xDE, 0x84, 0xE4, 0x84, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xA5, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED,
        0x27, 0xED, 0x47, 0xED, 0x48, 0xED, 0xE7, 0xDC, 0x82, 0x41, 0x00, 0x00, 0xA2, 0x49, 0x28, 0xDD, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xE9, 0xCC, 0x22, 0x31, 0x00, 0x00, 0x85, 0x6A, 0xC9, 0xC4, 0x00, 0x00, 0x00, 0x00, 0x65, 0x62, 0x0C, 0xF6,
        0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF,
        0xF2, 0xF6, 0x93, 0xC5, 0x14, 0xAD, 0xF4, 0xAC, 0x14, 0xB5, 0xFB, 0xE6, 0x9E, 0xF7, 0x96, 0xC5, 0xCF, 0x93, 0xCF, 0x93, 0x75, 0xC5, 0xFF, 0xFF,
  /* RLE: 040 Pixels @ 091,032*/ 40, 0xFF, 0xFF,
  /* ABS: 089 Pixels @ 003,033*/ 0, 89, 0x14, 0xBD, 0xCF, 0x93, 0x9A, 0xDE, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xDB, 0xDE, 0x79, 0xD6, 0x59, 0xD6, 0x18, 0xCE, 0xF7, 0xC5, 0xB6, 0xC5, 0x75, 0xBD,
        0x54, 0xBD, 0xB6, 0xC5, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0x13, 0xDE, 0xA4, 0xE4, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED,
        0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x68, 0xED, 0xE4, 0x7A, 0x00, 0x00, 0xA1, 0x18, 0x47, 0xB4, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xAA, 0xED, 0x84, 0x6A, 0x00, 0x00, 0x81, 0x10, 0x00, 0x00, 0x00, 0x00, 0x85, 0x62, 0x2C, 0xF6, 0x2D, 0xF6,
        0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x32, 0xFF,
        0x12, 0xF7, 0xF3, 0xCD, 0xF4, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0xBA, 0xDE, 0x9E, 0xF7, 0x38, 0xCE, 0xCF, 0x9B, 0xCF, 0x93, 0xF3, 0xB4, 0xDF, 0xFF,
  /* RLE: 039 Pixels @ 092,033*/ 39, 0xFF, 0xFF,
  /* ABS: 007 Pixels @ 003,034*/ 0, 7, 0x38, 0xD6, 0xCF, 0x93, 0xD7, 0xC5, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7,
  /* RLE: 004 Pixels @ 010,034*/ 4, 0x1C, 0xE7,
  /* ABS: 079 Pixels @ 014,034*/ 0, 79, 0xBA, 0xDE, 0x79, 0xD6, 0x59, 0xD6, 0x18, 0xCE, 0xF7, 0xC5, 0xB6, 0xC5, 0x75, 0xBD, 0x54, 0xBD, 0xD6, 0xC5, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xF2, 0xE5, 0xA4, 0xE4, 0xA4, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC,
        0xC5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x47, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x69, 0xED, 0x69, 0xED, 0x47, 0xB4, 0xA1, 0x18, 0x00, 0x00, 0xE5, 0x7A, 0xAA, 0xED, 0xCA, 0xED,
        0xCB, 0xED, 0xCB, 0xED, 0x08, 0xA4, 0x60, 0x10, 0x00, 0x00, 0x00, 0x00, 0x69, 0xB4, 0x2D, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6,
        0xD0, 0xF6, 0xD0, 0xF6, 0xD1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xDE, 0xF3, 0xAC, 0xF3, 0xAC, 0xD3, 0xAC, 0x59, 0xD6, 0x9E, 0xF7, 0x9A, 0xDE, 0x10, 0x9C, 0xCF, 0x93,
        0x92, 0xAC, 0x9E, 0xF7,
  /* RLE: 038 Pixels @ 093,034*/ 38, 0xFF, 0xFF,
  /* ABS: 091 Pixels @ 003,035*/ 0, 91, 0x3C, 0xEF, 0xCF, 0x93, 0xF3, 0xB4, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xBA, 0xDE, 0x79, 0xD6, 0x59, 0xD6, 0x18, 0xCE, 0xF7, 0xCD,
        0xB6, 0xC5, 0x75, 0xC5, 0x34, 0xBD, 0xB6, 0xC5, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xF1, 0xE5, 0xC5, 0xE4, 0xC5, 0xE4, 0xC5, 0xEC, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED,
        0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x29, 0xDD, 0x03, 0x52, 0x00, 0x00, 0x01, 0x29, 0x07, 0xAC, 0xCB, 0xED, 0xEB, 0xED, 0xEB, 0xF5, 0x4A, 0xD5, 0x42, 0x31, 0x00, 0x00, 0x64, 0x5A, 0xEC, 0xED, 0x2D, 0xF6,
        0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF,
        0x32, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0xB3, 0xE6, 0xF3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF7, 0xC5, 0x9E, 0xF7, 0xFB, 0xE6, 0x51, 0xA4, 0xCF, 0x93, 0x51, 0xA4, 0x7D, 0xF7,
  /* RLE: 037 Pixels @ 094,035*/ 37, 0xFF, 0xFF,
  /* ABS: 092 Pixels @ 003,036*/ 0, 92, 0xFF, 0xFF, 0x10, 0x9C, 0x10, 0x9C, 0x1C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0xBA, 0xDE, 0x79, 0xD6, 0x59, 0xD6, 0x38, 0xCE,
        0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xC5, 0x34, 0xBD, 0xB6, 0xC5, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xD0, 0xE5, 0xC5, 0xEC, 0xC5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED,
        0x48, 0xED, 0x68, 0xED, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x8A, 0xED, 0xAA, 0xED, 0xC8, 0xC4, 0x03, 0x52, 0x40, 0x08, 0x60, 0x10, 0x24, 0x5A, 0x66, 0x8B, 0x68, 0xB4, 0x4A, 0xD5, 0xC5, 0x6A, 0x00, 0x00, 0x02, 0x29, 0x2B, 0xCD,
        0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF,
        0x33, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x73, 0xFF, 0xF4, 0xEE, 0x13, 0xB5, 0xD3, 0xAC, 0xD3, 0xAC, 0x96, 0xBD, 0x7D, 0xEF, 0x5D, 0xEF, 0x92, 0xAC, 0xCF, 0x93, 0x30, 0xA4, 0x3C, 0xEF,
  /* RLE: 037 Pixels @ 095,036*/ 37, 0xFF, 0xFF,
  /* ABS: 092 Pixels @ 004,037*/ 0, 92, 0x34, 0xBD, 0xCF, 0x93, 0x59, 0xD6, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0xDB, 0xDE, 0x9A, 0xDE, 0x59, 0xD6, 0x38, 0xD6,
        0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xBD, 0xB6, 0xC5, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xAF, 0xE5, 0xE5, 0xEC, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x47, 0xED, 0x48, 0xED, 0x48, 0xED,
        0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xAA, 0xED, 0x28, 0xAC, 0x64, 0x62, 0x22, 0x29, 0x20, 0x08, 0x00, 0x00, 0x40, 0x08, 0x20, 0x08, 0x00, 0x00, 0x40, 0x08, 0x08, 0x9C,
        0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x53, 0xFF,
        0x53, 0xFF, 0x53, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0xF4, 0xEE, 0xF3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0x55, 0xB5, 0x5D, 0xEF, 0x7D, 0xEF, 0xD3, 0xAC, 0xCF, 0x93, 0x10, 0x9C, 0xFB, 0xE6,
  /* RLE: 036 Pixels @ 096,037*/ 36, 0xFF, 0xFF,
  /* ABS: 093 Pixels @ 004,038*/ 0, 93, 0x9A, 0xDE, 0xCF, 0x93, 0x34, 0xB5, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xEF, 0x5D, 0xEF, 0xDB, 0xE6, 0x9A, 0xDE, 0x79, 0xD6,
        0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xBD, 0xB6, 0xC5, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xAE, 0xE5, 0xE6, 0xEC, 0xE6, 0xEC, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED,
        0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xCB, 0xED, 0xEB, 0xED, 0xEB, 0xF5, 0xEC, 0xF5, 0xEC, 0xED, 0x2A, 0xCD, 0x48, 0xA4, 0x26, 0x7B, 0x24, 0x52, 0xC1, 0x20, 0x00, 0x00,
        0x65, 0x5A, 0x2D, 0xEE, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xFF,
        0x53, 0xFF, 0x53, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x94, 0xFF, 0xF4, 0xEE, 0xF3, 0xAC, 0xD3, 0xAC, 0xB3, 0xAC, 0x14, 0xB5, 0x3C, 0xEF, 0x7D, 0xEF, 0xF3, 0xB4, 0xCF, 0x93, 0xEF, 0x9B, 0xDB, 0xE6,
  /* RLE: 035 Pixels @ 097,038*/ 35, 0xFF, 0xFF,
  /* ABS: 094 Pixels @ 004,039*/ 0, 94, 0xDF, 0xFF, 0xEF, 0x9B, 0x10, 0x9C, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0xFB, 0xE6, 0xBA, 0xDE,
        0x79, 0xD6, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xB5, 0xB6, 0xC5, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x8E, 0xE5, 0x06, 0xED, 0x06, 0xED, 0x07, 0xED, 0x27, 0xED, 0x27, 0xED, 0x47, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED,
        0x69, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xCB, 0xED, 0xEB, 0xF5, 0xEB, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x2D, 0xEE, 0xE6, 0x72,
        0x00, 0x00, 0x22, 0x29, 0x6C, 0xCD, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xD1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xFF,
        0x53, 0xFF, 0x73, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x94, 0xFF, 0x95, 0xFF, 0xF4, 0xEE, 0xF3, 0xAC, 0xB3, 0xAC, 0xB3, 0xA4, 0xF4, 0xAC, 0x3C, 0xE7, 0x9E, 0xF7, 0x34, 0xBD, 0xCF, 0x93, 0xEF, 0x9B, 0x9A, 0xDE,
  /* RLE: 035 Pixels @ 098,039*/ 35, 0xFF, 0xFF,
  /* ABS: 094 Pixels @ 005,040*/ 0, 94, 0xF3, 0xB4, 0xCF, 0x93, 0x59, 0xD6, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0xFB, 0xE6, 0xBA, 0xDE,
        0x79, 0xD6, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xB5, 0x96, 0xC5, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0xD0, 0xE5, 0x06, 0xED, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x69, 0xED,
        0x89, 0xED, 0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xEB, 0xED, 0xEB, 0xF5, 0xEC, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0xCA, 0xB4,
        0x81, 0x10, 0x40, 0x08, 0x65, 0x5A, 0x88, 0x83, 0x2C, 0xBD, 0xB0, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x53, 0xFF,
        0x74, 0xFF, 0x74, 0xFF, 0x94, 0xFF, 0x94, 0xFF, 0x95, 0xFF, 0xB5, 0xFF, 0xF5, 0xE6, 0xD3, 0xAC, 0xB3, 0xA4, 0xB2, 0xA4, 0xF3, 0xAC, 0x1C, 0xE7, 0x9E, 0xF7, 0x75, 0xBD, 0xCF, 0x93, 0xCF, 0x9B, 0x38, 0xD6,
  /* RLE: 034 Pixels @ 099,040*/ 34, 0xFF, 0xFF,
  /* ABS: 058 Pixels @ 005,041*/ 0, 58, 0x59, 0xD6, 0xCF, 0x93, 0x55, 0xBD, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x9E, 0xF7, 0x9E, 0xF7, 0xFB, 0xE6,
        0x9A, 0xDE, 0x59, 0xD6, 0x18, 0xD6, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0x75, 0xBD, 0x9A, 0xD6, 0x9A, 0xDE, 0x9A, 0xDE, 0xF2, 0xDD, 0x27, 0xED, 0x27, 0xED, 0x27, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x68, 0xED, 0x69, 0xED,
        0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xCB, 0xED, 0xEB, 0xF5, 0xEB, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6,
        0xE2, 0x20,
  /* RLE: 004 Pixels @ 063,041*/ 4, 0x00, 0x00,
  /* ABS: 033 Pixels @ 067,041*/ 0, 33, 0xA4, 0x39, 0x2C, 0xBD, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xF7, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x73, 0xFF, 0x74, 0xFF, 0x74, 0xFF,
        0x94, 0xFF, 0x94, 0xFF, 0x95, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0x54, 0xD6, 0xB3, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0xD3, 0xAC, 0xFB, 0xE6, 0x9E, 0xF7, 0xB6, 0xC5, 0xCF, 0x93, 0xCF, 0x93, 0xF7, 0xCD,
  /* RLE: 033 Pixels @ 100,041*/ 33, 0xFF, 0xFF,
  /* ABS: 058 Pixels @ 005,042*/ 0, 58, 0x9E, 0xF7, 0xCF, 0x93, 0x10, 0x9C, 0xFB, 0xDE, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x9E, 0xF7, 0x9E, 0xF7, 0x7D, 0xF7,
        0xFB, 0xE6, 0x9A, 0xDE, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0x34, 0xBD, 0x79, 0xD6, 0x9A, 0xDE, 0x9A, 0xD6, 0x14, 0xDE, 0x28, 0xED, 0x47, 0xED, 0x48, 0xED, 0x48, 0xED, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED,
        0x89, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xEB, 0xED, 0xEB, 0xF5, 0xEC, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6,
        0xE6, 0x72,
  /* RLE: 005 Pixels @ 063,042*/ 5, 0x00, 0x00,
  /* ABS: 034 Pixels @ 068,042*/ 0, 34, 0x00, 0x00, 0x8B, 0xA4, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x94, 0xFF,
        0x95, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0xB6, 0xFF, 0x73, 0xBD, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0xB3, 0xA4, 0xDB, 0xDE, 0x9E, 0xF7, 0xF7, 0xCD, 0xCF, 0x93, 0xCF, 0x93, 0xB6, 0xC5, 0xFF, 0xFF,
  /* RLE: 032 Pixels @ 102,042*/ 32, 0xFF, 0xFF,
  /* ABS: 058 Pixels @ 006,043*/ 0, 58, 0xF3, 0xB4, 0xCF, 0x93, 0x18, 0xCE, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x9E, 0xF7, 0x9E, 0xF7, 0x9E, 0xF7, 0x7D, 0xF7,
        0xFB, 0xE6, 0x9A, 0xDE, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0x14, 0xB5, 0x59, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x35, 0xDE, 0x69, 0xED, 0x48, 0xED, 0x68, 0xED, 0x68, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED,
        0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xCB, 0xED, 0xEB, 0xED, 0xEB, 0xF5, 0xEC, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6,
        0x25, 0x52,
  /* RLE: 005 Pixels @ 064,043*/ 5, 0x00, 0x00,
  /* ABS: 034 Pixels @ 069,043*/ 0, 34, 0x81, 0x10, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x94, 0xFF, 0x94, 0xFF, 0x95, 0xFF,
        0xB5, 0xFF, 0xB5, 0xFF, 0xD6, 0xFF, 0xD6, 0xFF, 0x76, 0xF7, 0xD3, 0xAC, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xBA, 0xDE, 0x9E, 0xF7, 0x38, 0xD6, 0xCF, 0x93, 0xCF, 0x93, 0x55, 0xBD, 0xFF, 0xFF,
  /* RLE: 031 Pixels @ 103,043*/ 31, 0xFF, 0xFF,
  /* ABS: 098 Pixels @ 006,044*/ 0, 98, 0xBA, 0xDE, 0xCF, 0x93, 0xB2, 0xAC, 0x1C, 0xE7, 0xFB, 0xE6, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x9E, 0xF7, 0x9E, 0xF7, 0x9E, 0xF7, 0x7D, 0xF7,
        0x7D, 0xEF, 0xDB, 0xE6, 0x9A, 0xDE, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x54, 0xBD, 0x13, 0xB5, 0xF3, 0xB4, 0x38, 0xCE, 0x9A, 0xD6, 0x79, 0xD6, 0x58, 0xD6, 0x8C, 0xE5, 0x68, 0xED, 0x69, 0xED, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED,
        0xAA, 0xED, 0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xCB, 0xED, 0xEB, 0xF5, 0xEB, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6,
        0x8F, 0xF6, 0x2C, 0xC5, 0x25, 0x52, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x22, 0x29, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x73, 0xFF, 0x74, 0xFF, 0x74, 0xFF,
        0x94, 0xFF, 0x95, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0xD6, 0xFF, 0xD6, 0xFF, 0xD6, 0xFF, 0xF4, 0xCD, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x9A, 0xD6, 0x9E, 0xF7, 0x79, 0xD6, 0xCF, 0x9B, 0xCF, 0x93, 0x34, 0xBD,
        0xFF, 0xFF,
  /* RLE: 030 Pixels @ 104,044*/ 30, 0xFF, 0xFF,
  /* ABS: 099 Pixels @ 006,045*/ 0, 99, 0xFF, 0xFF, 0x30, 0xA4, 0xCF, 0x93, 0x9A, 0xD6, 0xFB, 0xE6, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x9E, 0xF7, 0x9E, 0xF7, 0x7D, 0xF7, 0x7D, 0xF7,
        0x7D, 0xF7, 0x5D, 0xEF, 0xDB, 0xE6, 0x9A, 0xDE, 0x58, 0xD6, 0x17, 0xCE, 0xD6, 0xC5, 0x95, 0xC5, 0x54, 0xBD, 0x13, 0xB5, 0xF3, 0xB4, 0x18, 0xCE, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0xCF, 0xE5, 0x69, 0xED, 0x89, 0xED, 0x89, 0xED, 0x89, 0xED,
        0xAA, 0xED, 0xAA, 0xED, 0xCA, 0xED, 0xCA, 0xED, 0xCB, 0xED, 0xEB, 0xED, 0xEB, 0xF5, 0xEC, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6,
        0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0x0E, 0xDE, 0x0C, 0xB5, 0x4D, 0xBD, 0xB0, 0xEE, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x94, 0xFF,
        0x94, 0xFF, 0x95, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0xD6, 0xFF, 0xD6, 0xFF, 0xF6, 0xFF, 0xD4, 0xC5, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x79, 0xD6, 0x9E, 0xF7, 0x9A, 0xDE, 0xEF, 0x9B, 0xCF, 0x93,
        0x34, 0xBD, 0xFF, 0xFF,
  /* RLE: 030 Pixels @ 105,045*/ 30, 0xFF, 0xFF,
  /* ABS: 099 Pixels @ 007,046*/ 0, 99, 0xD7, 0xCD, 0xCF, 0x93, 0x55, 0xBD, 0xFB, 0xDE, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x9E, 0xF7, 0x7D, 0xF7, 0x7D, 0xF7, 0x7D, 0xF7, 0x7D, 0xEF,
        0x7D, 0xEF, 0x5D, 0xEF, 0xDB, 0xE6, 0x99, 0xDE, 0x58, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xC5, 0x34, 0xBD, 0xF3, 0xB4, 0xD2, 0xAC, 0xD7, 0xC5, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x13, 0xDE, 0x8A, 0xED, 0x89, 0xED, 0xAA, 0xED, 0xAA, 0xED,
        0xEC, 0xED, 0xEC, 0xED, 0x0D, 0xF6, 0xEC, 0xED, 0xEB, 0xF5, 0xEB, 0xF5, 0x0C, 0xF6, 0x0C, 0xF6, 0x0C, 0xF6, 0x2C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6,
        0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x11, 0xF7, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x32, 0xFF, 0x33, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x95, 0xFF, 0x95, 0xFF,
        0x95, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0xD6, 0xFF, 0xD6, 0xFF, 0xD6, 0xFF, 0xD5, 0xE6, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x72, 0xA4, 0x72, 0xA4, 0x59, 0xCE, 0x9E, 0xF7, 0xBA, 0xDE, 0xEF, 0x9B, 0xCF, 0x93,
        0x34, 0xBD, 0xFF, 0xFF,
  /* RLE: 029 Pixels @ 106,046*/ 29, 0xFF, 0xFF,
  /* ABS: 100 Pixels @ 007,047*/ 0, 100, 0x9E, 0xF7, 0xCF, 0x93, 0x10, 0x9C, 0xFB, 0xDE, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xF7, 0x7D, 0xF7, 0x7D, 0xF7, 0x7D, 0xEF, 0x7D, 0xEF,
        0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0xDB, 0xE6, 0x79, 0xDE, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xBD, 0xF3, 0xB4, 0xB2, 0xAC, 0xB6, 0xC5, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x36, 0xD6, 0xAC, 0xE5, 0xAA, 0xED, 0xAA, 0xED,
        0xCB, 0xED, 0x2E, 0xF6, 0x2E, 0xF6, 0x4F, 0xF6, 0x4F, 0xF6, 0x4E, 0xF6, 0x2D, 0xF6, 0x0D, 0xF6, 0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6,
        0xB0, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0xF1, 0xF6, 0x12, 0xFF, 0x12, 0xFF, 0x33, 0xFF, 0x33, 0xFF, 0x54, 0xFF, 0x54, 0xFF, 0x75, 0xFF, 0x75, 0xFF, 0x75, 0xFF, 0x95, 0xFF, 0x95, 0xFF, 0x96, 0xFF, 0x95, 0xFF,
        0xB5, 0xFF, 0xB5, 0xFF, 0xB5, 0xFF, 0xD6, 0xFF, 0xB6, 0xFF, 0x54, 0xD6, 0xB3, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x72, 0xA4, 0x72, 0xA4, 0x71, 0x9C, 0x18, 0xCE, 0x9E, 0xF7, 0x9A, 0xDE, 0xEF, 0x9B,
        0xCF, 0x93, 0x34, 0xBD, 0xFF, 0xFF,
  /* RLE: 029 Pixels @ 107,047*/ 29, 0xFF, 0xFF,
  /* ABS: 100 Pixels @ 008,048*/ 0, 100, 0x14, 0xB5, 0xCF, 0x93, 0xB6, 0xC5, 0xFB, 0xE6, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xF7, 0x7D, 0xF7, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF,
        0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0xDB, 0xE6, 0x79, 0xDE, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xBD, 0xF3, 0xB4, 0xB2, 0xAC, 0x75, 0xBD, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x58, 0xD6, 0xF1, 0xDD, 0xAA, 0xED, 0xCA, 0xED,
        0xCB, 0xED, 0x4F, 0xF6, 0x70, 0xF6, 0x70, 0xF6, 0x71, 0xF6, 0x91, 0xF6, 0x91, 0xF6, 0x91, 0xF6, 0x70, 0xF6, 0x70, 0xF6, 0x6F, 0xF6, 0x6F, 0xF6, 0x8F, 0xF6, 0x6F, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xB0, 0xF6, 0xB0, 0xF6, 0xD1, 0xF6, 0xD1, 0xF6,
        0xF2, 0xF6, 0xF2, 0xF6, 0x13, 0xF7, 0x13, 0xFF, 0x34, 0xFF, 0x34, 0xFF, 0x35, 0xFF, 0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF, 0x75, 0xFF, 0x75, 0xFF, 0x76, 0xFF, 0x76, 0xFF, 0x96, 0xFF, 0x96, 0xFF, 0x95, 0xFF, 0x95, 0xFF, 0x95, 0xFF, 0xB5, 0xFF,
        0xB5, 0xFF, 0xB5, 0xFF, 0xD5, 0xE6, 0x33, 0xB5, 0xB3, 0xAC, 0xB3, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x72, 0xA4, 0x72, 0xA4, 0x71, 0x9C, 0x71, 0x9C, 0xF8, 0xCD, 0x9E, 0xF7, 0x9A, 0xDE, 0xCF, 0x9B,
        0xCF, 0x93, 0x75, 0xBD, 0xFF, 0xFF,
  /* RLE: 028 Pixels @ 108,048*/ 28, 0xFF, 0xFF,
  /* ABS: 100 Pixels @ 008,049*/ 0, 100, 0x1C, 0xEF, 0xCF, 0x93, 0x10, 0x9C, 0xFB, 0xDE, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF,
        0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF, 0xDB, 0xE6, 0x79, 0xDE, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xBD, 0xF3, 0xB4, 0xB2, 0xAC, 0x34, 0xBD, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x16, 0xD6, 0xCD, 0xED,
        0xCB, 0xED, 0xEC, 0xED, 0x70, 0xF6, 0x92, 0xF6, 0xB2, 0xF6, 0xB2, 0xF6, 0xB3, 0xF6, 0xD3, 0xF6, 0xD3, 0xF6, 0xD3, 0xF6, 0xD3, 0xF6, 0xD3, 0xF6, 0xF4, 0xF6, 0xF4, 0xF6, 0xF4, 0xF6, 0xF4, 0xF6, 0x14, 0xF7, 0x14, 0xFF, 0x14, 0xFF, 0x15, 0xFF,
        0x35, 0xFF, 0x35, 0xFF, 0x35, 0xFF, 0x35, 0xFF, 0x55, 0xFF, 0x56, 0xFF, 0x56, 0xFF, 0x56, 0xFF, 0x76, 0xFF, 0x76, 0xFF, 0x76, 0xFF, 0x77, 0xFF, 0x97, 0xFF, 0x96, 0xFF, 0x75, 0xFF, 0x74, 0xFF, 0x94, 0xFF, 0x95, 0xFF, 0xB5, 0xFF, 0x35, 0xEF,
        0x34, 0xD6, 0x13, 0xB5, 0xD3, 0xAC, 0xD3, 0xAC, 0xB3, 0xAC, 0xB3, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x72, 0xA4, 0x72, 0xA4, 0x71, 0x9C, 0x71, 0x9C, 0x51, 0x9C, 0x18, 0xCE, 0x9E, 0xF7, 0x79, 0xDE,
        0xCF, 0x9B, 0xCF, 0x93, 0x96, 0xC5,
  /* RLE: 029 Pixels @ 108,049*/ 29, 0xFF, 0xFF,
  /* ABS: 100 Pixels @ 009,050*/ 0, 100, 0x14, 0xB5, 0xCF, 0x93, 0x96, 0xBD, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF,
        0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0xDB, 0xE6, 0x79, 0xD6, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xB5, 0xF3, 0xB4, 0xB2, 0xAC, 0x13, 0xB5, 0x59, 0xCE, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0xF1, 0xDD,
        0xEB, 0xED, 0xEB, 0xF5, 0x4E, 0xF6, 0xB3, 0xF6, 0xD4, 0xF6, 0xF4, 0xF6, 0xF4, 0xF6, 0xF4, 0xF6, 0xF5, 0xF6, 0xF5, 0xF6, 0x15, 0xF7, 0x15, 0xF7, 0x15, 0xFF, 0x15, 0xFF, 0x15, 0xFF, 0x35, 0xFF, 0x36, 0xFF, 0x36, 0xFF, 0x36, 0xFF, 0x36, 0xFF,
        0x56, 0xFF, 0x56, 0xFF, 0x57, 0xFF, 0x57, 0xFF, 0x77, 0xFF, 0x77, 0xFF, 0x77, 0xFF, 0x77, 0xFF, 0x77, 0xFF, 0x76, 0xFF, 0x75, 0xFF, 0x54, 0xFF, 0x74, 0xFF, 0x74, 0xFF, 0x94, 0xFF, 0x74, 0xF7, 0x94, 0xDE, 0x94, 0xBD, 0xF3, 0xAC, 0xD3, 0xAC,
        0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xB3, 0xAC, 0xB3, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x72, 0xA4, 0x72, 0xA4, 0x71, 0x9C, 0x71, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x38, 0xCE, 0x9E, 0xF7, 0x79, 0xD6,
        0xCF, 0x93, 0xCF, 0x93, 0x59, 0xD6,
  /* RLE: 028 Pixels @ 109,050*/ 28, 0xFF, 0xFF,
  /* ABS: 101 Pixels @ 009,051*/ 0, 101, 0x3C, 0xEF, 0xCF, 0x93, 0x10, 0x9C, 0xDB, 0xDE, 0x1C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF,
        0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7, 0xDA, 0xDE, 0x79, 0xD6, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xB5, 0xD3, 0xB4, 0x92, 0xAC, 0xD3, 0xAC, 0x38, 0xCE, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE,
        0x16, 0xD6, 0xEF, 0xE5, 0x0C, 0xF6, 0x0C, 0xF6, 0x4F, 0xF6, 0xB2, 0xF6, 0xF5, 0xF6, 0x16, 0xFF, 0x16, 0xFF, 0x16, 0xFF, 0x36, 0xFF, 0x36, 0xFF, 0x37, 0xFF, 0x37, 0xFF, 0x37, 0xFF, 0x57, 0xFF, 0x57, 0xFF, 0x57, 0xFF, 0x57, 0xFF, 0x57, 0xFF,
        0x57, 0xFF, 0x78, 0xFF, 0x78, 0xFF, 0x78, 0xFF, 0x77, 0xFF, 0x56, 0xFF, 0x55, 0xFF, 0x54, 0xFF, 0x33, 0xFF, 0x53, 0xFF, 0x53, 0xFF, 0x53, 0xF7, 0xD4, 0xE6, 0x34, 0xD6, 0x94, 0xBD, 0x14, 0xAD, 0xF4, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xD3, 0xAC,
        0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xB3, 0xAC, 0xB3, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x72, 0xA4, 0x72, 0xA4, 0x71, 0x9C, 0x71, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0xF4, 0xAC, 0xFB, 0xE6, 0xDF, 0xFF,
        0x5D, 0xEF, 0x51, 0xA4, 0xEF, 0x9B, 0x7D, 0xF7,
  /* RLE: 028 Pixels @ 110,051*/ 28, 0xFF, 0xFF,
  /* ABS: 093 Pixels @ 010,052*/ 0, 93, 0x34, 0xBD, 0xCF, 0x93, 0x96, 0xBD, 0x1C, 0xE7, 0x1C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF,
        0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0xBA, 0xDE, 0x59, 0xD6, 0x18, 0xD6, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0xD3, 0xB4, 0x92, 0xAC, 0xB2, 0xAC, 0x18, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE,
        0x18, 0xCE, 0x15, 0xD6, 0x0E, 0xEE, 0x0C, 0xF6, 0x2D, 0xF6, 0x2D, 0xF6, 0x4E, 0xF6, 0x90, 0xF6, 0xD3, 0xF6, 0xF4, 0xF6, 0x14, 0xF7, 0x15, 0xFF, 0x36, 0xFF, 0x36, 0xFF, 0x36, 0xFF, 0x36, 0xFF, 0x35, 0xFF, 0x35, 0xFF, 0x35, 0xFF, 0x34, 0xFF,
        0x13, 0xFF, 0x12, 0xF7, 0x12, 0xFF, 0x12, 0xFF, 0x12, 0xFF, 0x32, 0xFF, 0x13, 0xF7, 0xB3, 0xE6, 0x54, 0xD6, 0xB4, 0xC5, 0x34, 0xB5, 0x14, 0xB5, 0x14, 0xB5, 0x14, 0xAD, 0x14, 0xAD, 0xF4, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC,
        0xD3, 0xAC, 0xD3, 0xAC, 0xB3, 0xAC, 0xB3, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x72, 0xA4, 0xD3, 0xAC, 0x96, 0xBD, 0x59, 0xD6, 0x1C, 0xEF, 0xDF, 0xFF,
  /* RLE: 004 Pixels @ 103,052*/ 4, 0xFF, 0xFF,
  /* ABS: 003 Pixels @ 107,052*/ 0, 3, 0xBE, 0xF7, 0xEF, 0x9B, 0xD7, 0xCD,
  /* RLE: 028 Pixels @ 110,052*/ 28, 0xFF, 0xFF,
  /* ABS: 088 Pixels @ 010,053*/ 0, 88, 0x5D, 0xEF, 0xCF, 0x9B, 0xEF, 0x9B, 0x9A, 0xD6, 0x3C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF,
        0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xBA, 0xDE, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0xD3, 0xAC, 0x92, 0xAC, 0x92, 0xAC, 0xF7, 0xC5, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE,
        0x18, 0xCE, 0x18, 0xCE, 0x17, 0xCE, 0x13, 0xDE, 0x2D, 0xEE, 0x2D, 0xF6, 0x4D, 0xF6, 0x4E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x6E, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0x8F, 0xF6, 0xAF, 0xF6, 0xB0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xD0, 0xF6, 0xF1, 0xF6,
        0xF1, 0xF6, 0xD2, 0xEE, 0x93, 0xE6, 0x33, 0xD6, 0xD4, 0xCD, 0x74, 0xBD, 0x55, 0xB5, 0x35, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x14, 0xB5, 0x14, 0xB5, 0x14, 0xAD, 0x14, 0xAD, 0xF4, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC,
        0xD3, 0xAC, 0xD3, 0xAC, 0xB3, 0xAC, 0xB3, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0xD3, 0xAC, 0x75, 0xBD, 0x38, 0xCE, 0xDB, 0xE6, 0x9E, 0xF7,
  /* RLE: 006 Pixels @ 098,053*/ 6, 0xFF, 0xFF,
  /* ABS: 006 Pixels @ 104,053*/ 0, 6, 0xFF, 0xFF, 0x7D, 0xF7, 0x5D, 0xEF, 0xDF, 0xFF, 0x55, 0xBD, 0xF3, 0xB4,
  /* RLE: 029 Pixels @ 110,053*/ 29, 0xFF, 0xFF,
  /* ABS: 083 Pixels @ 011,054*/ 0, 83, 0x96, 0xC5, 0xCF, 0x93, 0x92, 0xAC, 0x1C, 0xE7, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF,
        0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xBA, 0xDE, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0xD3, 0xAC, 0x91, 0xAC, 0x71, 0xA4, 0xB6, 0xC5, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE,
        0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xF7, 0xCD, 0xF5, 0xD5, 0x13, 0xDE, 0x31, 0xE6, 0x50, 0xEE, 0x50, 0xEE, 0x50, 0xEE, 0x50, 0xEE, 0x51, 0xEE, 0x71, 0xEE, 0x51, 0xE6, 0x52, 0xDE, 0x33, 0xDE, 0xF4, 0xD5, 0xD4, 0xCD, 0xB5, 0xC5, 0x75, 0xBD,
        0x75, 0xBD, 0x75, 0xBD, 0x55, 0xBD, 0x55, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x35, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x14, 0xB5, 0x14, 0xB5, 0x14, 0xAD, 0x14, 0xAD, 0xF4, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC,
        0xD3, 0xAC, 0x55, 0xB5, 0xF7, 0xCD, 0xBA, 0xDE, 0x5D, 0xEF, 0xFF, 0xFF,
  /* RLE: 006 Pixels @ 094,054*/ 6, 0xFF, 0xFF,
  /* ABS: 010 Pixels @ 100,054*/ 0, 10, 0xFF, 0xFF, 0x3C, 0xEF, 0x79, 0xD6, 0xB7, 0xC5, 0x14, 0xAD, 0xF3, 0xAC, 0xF3, 0xAC, 0x55, 0xBD, 0xD7, 0xCD, 0x51, 0xA4,
  /* RLE: 029 Pixels @ 110,054*/ 29, 0xFF, 0xFF,
  /* ABS: 078 Pixels @ 011,055*/ 0, 78, 0xDF, 0xFF, 0x51, 0xA4, 0xCF, 0x93, 0x96, 0xC5, 0x3C, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7,
        0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xBA, 0xDE, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x54, 0xBD, 0x13, 0xB5, 0xD2, 0xAC, 0x91, 0xAC, 0x51, 0xA4, 0x75, 0xBD, 0x18, 0xCE, 0x18, 0xCE,
        0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xF7, 0xCD, 0xF7, 0xC5, 0xF7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xB7, 0xC5, 0xB6, 0xC5, 0xB6, 0xC5, 0xB6, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x76, 0xBD, 0x75, 0xBD,
        0x75, 0xBD, 0x75, 0xBD, 0x55, 0xBD, 0x55, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x35, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x14, 0xB5, 0x14, 0xB5, 0x14, 0xAD, 0x14, 0xAD, 0xF4, 0xAC, 0xF3, 0xAC, 0x34, 0xB5, 0xD7, 0xC5, 0x7A, 0xD6, 0x3C, 0xEF,
        0xDF, 0xFF,
  /* RLE: 007 Pixels @ 089,055*/ 7, 0xFF, 0xFF,
  /* ABS: 014 Pixels @ 096,055*/ 0, 14, 0xBE, 0xFF, 0xFC, 0xE6, 0x38, 0xCE, 0x76, 0xBD, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xB5, 0x51, 0xA4, 0x92, 0xAC,
  /* RLE: 030 Pixels @ 110,055*/ 30, 0xFF, 0xFF,
  /* ABS: 072 Pixels @ 012,056*/ 0, 72, 0xFB, 0xE6, 0xCF, 0x93, 0xEF, 0x9B, 0xBA, 0xDE, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7,
        0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xBA, 0xDE, 0x58, 0xD6, 0x17, 0xCE, 0xD6, 0xC5, 0x95, 0xC5, 0x54, 0xBD, 0x13, 0xB5, 0xD2, 0xAC, 0x91, 0xA4, 0x50, 0xA4, 0x14, 0xB5, 0x18, 0xCE, 0x18, 0xCE,
        0x18, 0xCE, 0xF8, 0xCD, 0xF7, 0xCD, 0xF7, 0xC5, 0xF7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xB7, 0xC5, 0xB6, 0xC5, 0xB6, 0xC5, 0xB6, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x76, 0xBD, 0x75, 0xBD, 0x75, 0xBD,
        0x75, 0xBD, 0x55, 0xBD, 0x55, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x35, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x14, 0xB5, 0x34, 0xB5, 0xB6, 0xC5, 0x59, 0xD6, 0xFB, 0xE6, 0x9E, 0xF7,
  /* RLE: 008 Pixels @ 084,056*/ 8, 0xFF, 0xFF,
  /* ABS: 018 Pixels @ 092,056*/ 0, 18, 0x9E, 0xF7, 0xDB, 0xDE, 0xF8, 0xCD, 0x35, 0xB5, 0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xB5, 0x14, 0xB5, 0x34, 0xB5, 0xCF, 0x93,
        0xB2, 0xAC,
  /* RLE: 031 Pixels @ 110,056*/ 31, 0xFF, 0xFF,
  /* ABS: 067 Pixels @ 013,057*/ 0, 67, 0x96, 0xC5, 0xCF, 0x93, 0x71, 0xA4, 0x3C, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7,
        0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0x58, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xC5, 0x34, 0xBD, 0xF3, 0xB4, 0xB2, 0xAC, 0x71, 0xA4, 0x30, 0xA4, 0xD3, 0xAC, 0xF8, 0xCD, 0x18, 0xCE,
        0xF8, 0xCD, 0xF7, 0xCD, 0xF7, 0xC5, 0xF7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xB7, 0xC5, 0xB6, 0xC5, 0xB6, 0xC5, 0xB6, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x76, 0xBD, 0x75, 0xBD, 0x75, 0xBD, 0x75, 0xBD,
        0x55, 0xBD, 0x55, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x35, 0xB5, 0x96, 0xBD, 0x38, 0xCE, 0xDB, 0xDE, 0x7D, 0xEF, 0xFF, 0xFF,
  /* RLE: 007 Pixels @ 080,057*/ 7, 0xFF, 0xFF,
  /* ABS: 023 Pixels @ 087,057*/ 0, 23, 0xFF, 0xFF, 0x5D, 0xEF, 0x9A, 0xDE, 0xB7, 0xC5, 0xF4, 0xAC, 0xB2, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xB5, 0x14, 0xB5,
        0x34, 0xB5, 0x34, 0xB5, 0x35, 0xB5, 0x34, 0xB5, 0xCF, 0x93, 0xD3, 0xB4,
  /* RLE: 031 Pixels @ 110,057*/ 31, 0xFF, 0xFF,
  /* ABS: 062 Pixels @ 013,058*/ 0, 62, 0xDF, 0xFF, 0x71, 0xA4, 0xCF, 0x93, 0x96, 0xC5, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7,
        0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xBD, 0xF3, 0xB4, 0xB2, 0xAC, 0x71, 0xA4, 0x30, 0xA4, 0x91, 0xA4, 0xF7, 0xC5,
        0xF8, 0xCD, 0xF7, 0xCD, 0xF7, 0xC5, 0xF7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xB7, 0xC5, 0xB6, 0xC5, 0xB6, 0xC5, 0xB6, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x76, 0xBD, 0x75, 0xBD, 0x75, 0xBD, 0x75, 0xBD,
        0x96, 0xBD, 0x18, 0xCE, 0xBA, 0xDE, 0x3C, 0xEF, 0xDF, 0xFF,
  /* RLE: 008 Pixels @ 075,058*/ 8, 0xFF, 0xFF,
  /* ABS: 027 Pixels @ 083,058*/ 0, 27, 0xDF, 0xFF, 0x1C, 0xEF, 0x59, 0xD6, 0x76, 0xBD, 0xB3, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0x14, 0xAD,
        0x14, 0xB5, 0x14, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x35, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x34, 0xB5, 0xCF, 0x93, 0xF3, 0xB4,
  /* RLE: 032 Pixels @ 110,058*/ 32, 0xFF, 0xFF,
  /* ABS: 056 Pixels @ 014,059*/ 0, 56, 0x1C, 0xEF, 0xCF, 0x9B, 0xCF, 0x93, 0x59, 0xD6, 0x5D, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6,
        0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xBD, 0xF3, 0xB4, 0xB2, 0xAC, 0x71, 0xA4, 0x30, 0x9C, 0x50, 0xA4, 0xB6, 0xC5,
        0xF7, 0xCD, 0xF7, 0xC5, 0xF7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xB7, 0xC5, 0xB6, 0xC5, 0xB6, 0xC5, 0xB6, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0xF8, 0xCD, 0x9A, 0xD6, 0x1C, 0xE7, 0x9E, 0xF7,
  /* RLE: 009 Pixels @ 070,059*/ 9, 0xFF, 0xFF,
  /* ABS: 031 Pixels @ 079,059*/ 0, 31, 0xBE, 0xFF, 0xFB, 0xE6, 0x18, 0xCE, 0x35, 0xB5, 0x92, 0xA4, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xB3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC,
        0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xB5, 0x14, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x35, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x55, 0xBD, 0x75, 0xBD, 0x35, 0xB5, 0xCF, 0x93, 0x14, 0xB5,
  /* RLE: 033 Pixels @ 110,059*/ 33, 0xFF, 0xFF,
  /* ABS: 051 Pixels @ 015,060*/ 0, 51, 0xF7, 0xCD, 0xCF, 0x93, 0xEF, 0x9B, 0xDB, 0xDE, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6,
        0xDB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xBD, 0xF3, 0xB4, 0xB2, 0xAC, 0x71, 0xA4, 0x30, 0x9C, 0x10, 0x9C, 0x96, 0xBD,
        0xF7, 0xC5, 0xF7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xB7, 0xC5, 0xB6, 0xC5, 0xB6, 0xC5, 0xF7, 0xC5, 0x79, 0xD6, 0xFB, 0xE6, 0x7D, 0xF7, 0xFF, 0xFF,
  /* RLE: 009 Pixels @ 066,060*/ 9, 0xFF, 0xFF,
  /* ABS: 035 Pixels @ 075,060*/ 0, 35, 0x9E, 0xF7, 0xBA, 0xDE, 0xD7, 0xC5, 0xF3, 0xAC, 0x71, 0x9C, 0x71, 0x9C, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC,
        0xF3, 0xAC, 0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xB5, 0x14, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x35, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x75, 0xBD, 0x75, 0xBD, 0x75, 0xBD, 0x96, 0xBD, 0x35, 0xB5, 0xCF, 0x93, 0x34, 0xBD,
  /* RLE: 034 Pixels @ 110,060*/ 34, 0xFF, 0xFF,
  /* ABS: 045 Pixels @ 016,061*/ 0, 45, 0x34, 0xBD, 0xCF, 0x93, 0x51, 0xA4, 0x1C, 0xE7, 0x3C, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE,
        0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x34, 0xB5, 0xF3, 0xB4, 0xB2, 0xAC, 0x71, 0xA4, 0x30, 0x9C, 0xEF, 0x9B, 0x55, 0xBD,
        0xF7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0x59, 0xD6, 0xDB, 0xDE, 0x5D, 0xEF, 0xDF, 0xFF,
  /* RLE: 009 Pixels @ 061,061*/ 9, 0xFF, 0xFF,
  /* ABS: 040 Pixels @ 070,061*/ 0, 40, 0xFF, 0xFF, 0x5D, 0xEF, 0x79, 0xD6, 0x96, 0xBD, 0xB2, 0xA4, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x71, 0xA4, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC,
        0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xB5, 0x14, 0xB5, 0x14, 0xB5, 0xF3, 0xAC, 0x92, 0xA4, 0x51, 0xA4, 0x30, 0x9C, 0x34, 0xB5, 0x75, 0xBD, 0x75, 0xBD, 0x76, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD,
        0x55, 0xB5, 0xCF, 0x93, 0x75, 0xBD,
  /* RLE: 034 Pixels @ 110,061*/ 34, 0xFF, 0xFF,
  /* ABS: 041 Pixels @ 016,062*/ 0, 41, 0xDF, 0xFF, 0x92, 0xAC, 0xCF, 0x93, 0xF3, 0xB4, 0x3C, 0xEF, 0x3C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE,
        0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x38, 0xD6, 0xF7, 0xCD, 0xB6, 0xC5, 0x75, 0xBD, 0x14, 0xB5, 0xD3, 0xB4, 0x92, 0xAC, 0x51, 0xA4, 0x10, 0x9C, 0xCF, 0x93,
        0x14, 0xB5, 0x79, 0xD6, 0x5D, 0xEF, 0xDF, 0xFF,
  /* RLE: 009 Pixels @ 057,062*/ 9, 0xFF, 0xFF,
  /* ABS: 030 Pixels @ 066,062*/ 0, 30, 0xDF, 0xFF, 0x1C, 0xE7, 0x38, 0xCE, 0x35, 0xB5, 0x71, 0x9C, 0x31, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x72, 0xA4, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4,
        0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xAD, 0xD3, 0xAC, 0x72, 0xA4, 0x30, 0x9C, 0xCF, 0x9B,
  /* RLE: 004 Pixels @ 096,062*/ 4, 0xCF, 0x93,
  /* ABS: 010 Pixels @ 100,062*/ 0, 10, 0xEF, 0x9B, 0x14, 0xB5, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0xB6, 0xBD, 0xB6, 0xC5, 0x55, 0xB5, 0xCF, 0x93, 0x96, 0xC5,
  /* RLE: 035 Pixels @ 110,062*/ 35, 0xFF, 0xFF,
  /* ABS: 038 Pixels @ 017,063*/ 0, 38, 0x7D, 0xF7, 0x10, 0x9C, 0xCF, 0x93, 0x96, 0xC5, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE,
        0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x38, 0xD6, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0xD3, 0xB4, 0x92, 0xAC, 0x51, 0xA4, 0x10, 0x9C, 0x18, 0xCE,
        0xFF, 0xFF,
  /* RLE: 007 Pixels @ 055,063*/ 7, 0xFF, 0xFF,
  /* ABS: 030 Pixels @ 062,063*/ 0, 30, 0xBE, 0xF7, 0xDB, 0xE6, 0xF7, 0xC5, 0xF4, 0xAC, 0x31, 0x9C, 0x30, 0x9C, 0x31, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x71, 0x9C, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4,
        0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0xF3, 0xAC, 0xB2, 0xAC, 0x71, 0xA4, 0x10, 0x9C, 0xCF, 0x93,
  /* RLE: 005 Pixels @ 092,063*/ 5, 0xCF, 0x93,
  /* ABS: 013 Pixels @ 097,063*/ 0, 13, 0xEF, 0x93, 0x10, 0x94, 0xEF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xD3, 0xAC, 0xB6, 0xC5, 0xB6, 0xC5, 0xB7, 0xC5, 0xD7, 0xC5, 0x55, 0xBD, 0xCF, 0x93, 0xB6, 0xC5,
  /* RLE: 036 Pixels @ 110,063*/ 36, 0xFF, 0xFF,
  /* ABS: 035 Pixels @ 018,064*/ 0, 35, 0xDB, 0xE6, 0xCF, 0x9B, 0xCF, 0x93, 0x38, 0xD6, 0x1C, 0xE7, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE,
        0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0xD3, 0xAC, 0x92, 0xAC, 0x96, 0xC5,
  /* RLE: 005 Pixels @ 053,064*/ 5, 0xFF, 0xFF,
  /* ABS: 029 Pixels @ 058,064*/ 0, 29, 0x7E, 0xF7, 0x9A, 0xDE, 0xB6, 0xC5, 0xB3, 0xAC, 0x10, 0x94, 0x10, 0x94, 0x30, 0x9C, 0x30, 0x9C, 0x31, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x71, 0x9C, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4,
        0x92, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xD3, 0xAC, 0x92, 0xA4, 0x51, 0xA4, 0x10, 0x9C,
  /* RLE: 006 Pixels @ 087,064*/ 6, 0xCF, 0x93,
  /* ABS: 017 Pixels @ 093,064*/ 0, 17, 0xEF, 0x93, 0x30, 0x94, 0xD3, 0xA4, 0x96, 0xB5, 0x59, 0xCE, 0xFB, 0xDE, 0x79, 0xCE, 0x30, 0x9C, 0xCF, 0x93, 0xCF, 0x93, 0x92, 0xA4, 0xB7, 0xC5, 0xD7, 0xC5, 0xF7, 0xC5, 0x55, 0xB5, 0xCF, 0x93, 0xD7, 0xCD,
  /* RLE: 037 Pixels @ 110,064*/ 37, 0xFF, 0xFF,
  /* ABS: 039 Pixels @ 019,065*/ 0, 39, 0x18, 0xD6, 0xCF, 0x93, 0x10, 0x9C, 0x9A, 0xD6, 0x1C, 0xE7, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE,
        0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x39, 0xD6, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x55, 0xBD, 0x14, 0xB5, 0xD3, 0xAC, 0x9E, 0xF7, 0xFF, 0xFF, 0xFF, 0xFF, 0x79, 0xD6,
        0x75, 0xBD, 0x92, 0xA4,
  /* RLE: 004 Pixels @ 058,065*/ 4, 0x10, 0x94,
  /* ABS: 021 Pixels @ 062,065*/ 0, 21, 0x30, 0x9C, 0x31, 0x9C, 0x31, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x72, 0xA4, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC,
        0xB3, 0xAC, 0x72, 0xA4, 0x51, 0x9C, 0xEF, 0x9B,
  /* RLE: 006 Pixels @ 083,065*/ 6, 0xCF, 0x93,
  /* ABS: 021 Pixels @ 089,065*/ 0, 21, 0xEF, 0x93, 0x30, 0x9C, 0xD3, 0xA4, 0x55, 0xAD, 0xD7, 0xBD, 0x38, 0xC6, 0x79, 0xCE, 0x9A, 0xD6, 0xBA, 0xD6, 0xFB, 0xDE, 0x1C, 0xE7, 0xBA, 0xD6, 0x92, 0x9C, 0xCF, 0x93, 0xCF, 0x93, 0x51, 0xA4, 0xB6, 0xC5,
        0xF8, 0xCD, 0x55, 0xBD, 0xCF, 0x93, 0xF7, 0xCD,
  /* RLE: 038 Pixels @ 110,065*/ 38, 0xFF, 0xFF,
  /* ABS: 035 Pixels @ 020,066*/ 0, 35, 0x55, 0xBD, 0xCF, 0x93, 0x30, 0x9C, 0x9A, 0xDE, 0xFB, 0xE6, 0xFB, 0xE6, 0xFB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6,
        0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x39, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0xD7, 0xCD, 0x96, 0xC5, 0x54, 0xBD, 0x96, 0xC5, 0xFF, 0xFF, 0xFF, 0xFF, 0xF3, 0xAC,
  /* RLE: 004 Pixels @ 055,066*/ 4, 0x10, 0x94,
  /* ABS: 020 Pixels @ 059,066*/ 0, 20, 0x30, 0x94, 0x30, 0x9C, 0x31, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x71, 0x9C, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xB3, 0xAC, 0x92, 0xA4,
        0x71, 0xA4, 0x30, 0x9C, 0xEF, 0x9B,
  /* RLE: 005 Pixels @ 079,066*/ 5, 0xCF, 0x93,
  /* ABS: 026 Pixels @ 084,066*/ 0, 26, 0xCF, 0x93, 0x10, 0x94, 0x51, 0x9C, 0xB2, 0x9C, 0x14, 0xA5, 0x55, 0xAD, 0x96, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0xD7, 0xBD, 0xF7, 0xBD, 0x18, 0xC6, 0x38, 0xC6, 0x59, 0xCE, 0x9A, 0xD6, 0xBA, 0xD6, 0xDB, 0xDE,
        0xBA, 0xD6, 0xF3, 0xA4, 0xCF, 0x93, 0xCF, 0x93, 0x10, 0x9C, 0x96, 0xC5, 0x35, 0xB5, 0xCF, 0x93, 0x38, 0xD6,
  /* RLE: 038 Pixels @ 110,066*/ 38, 0xFF, 0xFF,
  /* ABS: 055 Pixels @ 020,067*/ 0, 55, 0xFF, 0xFF, 0xF3, 0xB4, 0xCF, 0x93, 0x51, 0xA4, 0xBA, 0xDE, 0xFB, 0xE6, 0xDB, 0xE6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6,
        0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xD6, 0xC5, 0x95, 0xC5, 0x79, 0xD6, 0xFF, 0xFF, 0xFB, 0xE6, 0x10, 0x94, 0x10, 0x94, 0x10, 0x94,
        0x30, 0x9C, 0x30, 0x9C, 0x31, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x71, 0x9C, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0x92, 0xA4, 0x51, 0xA4, 0x10, 0x9C, 0xCF, 0x9B,
  /* RLE: 005 Pixels @ 075,067*/ 5, 0xCF, 0x93,
  /* ABS: 030 Pixels @ 080,067*/ 0, 30, 0xCF, 0x93, 0x10, 0x94, 0x51, 0x94, 0x92, 0x94, 0xD3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0x14, 0xA5, 0x34, 0xA5, 0x34, 0xA5, 0x55, 0xAD, 0x75, 0xAD, 0x75, 0xAD, 0x96, 0xB5, 0xB6, 0xB5, 0xD7, 0xBD, 0xF7, 0xBD,
        0x18, 0xC6, 0x38, 0xC6, 0x59, 0xCE, 0x79, 0xCE, 0x9A, 0xD6, 0x9A, 0xD6, 0x55, 0xAD, 0xEF, 0x93, 0xCF, 0x93, 0xCF, 0x9B, 0xB2, 0xAC, 0xCF, 0x93, 0x59, 0xD6,
  /* RLE: 039 Pixels @ 110,067*/ 39, 0xFF, 0xFF,
  /* ABS: 050 Pixels @ 021,068*/ 0, 50, 0xDF, 0xFF, 0x92, 0xAC, 0xCF, 0x93, 0x71, 0xA4, 0xBA, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6,
        0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF7, 0xCD, 0xB6, 0xC5, 0xBA, 0xDE, 0xFF, 0xFF, 0x75, 0xBD, 0x10, 0x94, 0x30, 0x9C, 0x31, 0x9C, 0x31, 0x9C,
        0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x71, 0xA4, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0x71, 0xA4, 0x51, 0x9C, 0x10, 0x9C, 0xCF, 0x93,
  /* RLE: 005 Pixels @ 071,068*/ 5, 0xCF, 0x93,
  /* ABS: 034 Pixels @ 076,068*/ 0, 34, 0xEF, 0x93, 0x10, 0x94, 0x51, 0x94, 0x92, 0x94, 0xB2, 0x94, 0xB2, 0x94, 0xB2, 0x94, 0xB2, 0x94, 0xD3, 0x9C, 0xD3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0x14, 0xA5, 0x34, 0xA5, 0x34, 0xA5,
        0x55, 0xAD, 0x75, 0xAD, 0x96, 0xB5, 0x96, 0xB5, 0xB6, 0xB5, 0xD7, 0xBD, 0xF7, 0xBD, 0x18, 0xC6, 0x38, 0xC6, 0x59, 0xCE, 0x79, 0xCE, 0xB6, 0xB5, 0x10, 0x94, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0x79, 0xDE,
  /* RLE: 040 Pixels @ 110,068*/ 40, 0xFF, 0xFF,
  /* ABS: 044 Pixels @ 022,069*/ 0, 44, 0x9E, 0xF7, 0x51, 0xA4, 0xCF, 0x93, 0xB2, 0xAC, 0xDB, 0xDE, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6,
        0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF7, 0xCD, 0xDB, 0xDE, 0xFF, 0xFF, 0x51, 0x9C, 0x31, 0x9C, 0x31, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C,
        0x71, 0x9C, 0x72, 0xA4, 0x72, 0xA4, 0x72, 0xA4, 0x51, 0x9C, 0x30, 0x9C, 0xEF, 0x9B,
  /* RLE: 006 Pixels @ 066,069*/ 6, 0xCF, 0x93,
  /* ABS: 004 Pixels @ 072,069*/ 0, 4, 0xEF, 0x93, 0x30, 0x94, 0x71, 0x94, 0x92, 0x94,
  /* RLE: 010 Pixels @ 076,069*/ 10, 0xB2, 0x94,
  /* ABS: 025 Pixels @ 086,069*/ 0, 25, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0x34, 0xA5, 0x34, 0xA5, 0x55, 0xAD, 0x55, 0xAD, 0x75, 0xAD, 0x96, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0xD7, 0xBD, 0xF7, 0xBD,
        0x18, 0xC6, 0x38, 0xC6, 0xD7, 0xBD, 0x51, 0x9C, 0xCF, 0x93, 0xCF, 0x93, 0x92, 0xAC, 0xDF, 0xFF,
  /* RLE: 040 Pixels @ 111,069*/ 40, 0xFF, 0xFF,
  /* ABS: 039 Pixels @ 023,070*/ 0, 39, 0x7D, 0xF7, 0x30, 0xA4, 0xCF, 0x93, 0xF3, 0xB4, 0xDB, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6,
        0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xFB, 0xE6, 0x5D, 0xEF, 0x51, 0x9C, 0x51, 0x9C, 0x51, 0x9C, 0x71, 0x9C, 0x71, 0x9C, 0x71, 0x9C, 0x51, 0x9C,
        0x10, 0x9C, 0xEF, 0x9B,
  /* RLE: 006 Pixels @ 062,070*/ 6, 0xCF, 0x93,
  /* ABS: 008 Pixels @ 068,070*/ 0, 8, 0x10, 0x9C, 0x71, 0x9C, 0xB2, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 012 Pixels @ 076,070*/ 12, 0xB2, 0x94,
  /* ABS: 024 Pixels @ 088,070*/ 0, 24, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0x14, 0xA5, 0x34, 0xA5, 0x34, 0xA5, 0x55, 0xAD, 0x75, 0xAD, 0x75, 0xAD, 0x96, 0xB5, 0xB6, 0xB5, 0xD7, 0xBD, 0xF7, 0xBD,
        0x18, 0xC6, 0xF7, 0xBD, 0x92, 0x9C, 0xCF, 0x93, 0xCF, 0x93, 0x71, 0xAC, 0x7D, 0xF7,
  /* RLE: 040 Pixels @ 112,070*/ 40, 0xFF, 0xFF,
  /* ABS: 034 Pixels @ 024,071*/ 0, 34, 0x3C, 0xEF, 0xEF, 0x9B, 0xCF, 0x93, 0x34, 0xBD, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6,
        0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x1C, 0xE7, 0xBA, 0xDE, 0x51, 0x9C, 0x71, 0x9C, 0x71, 0x9C, 0x72, 0xA4, 0xEF, 0x9B,
  /* RLE: 005 Pixels @ 058,071*/ 5, 0xCF, 0x93,
  /* ABS: 014 Pixels @ 063,071*/ 0, 14, 0xCF, 0x9B, 0x30, 0x9C, 0xB2, 0xA4, 0xF3, 0xA4, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 013 Pixels @ 077,071*/ 13, 0xB2, 0x94,
  /* ABS: 023 Pixels @ 090,071*/ 0, 23, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0x14, 0xA5, 0x34, 0xA5, 0x34, 0xA5, 0x55, 0xAD, 0x75, 0xAD, 0x96, 0xB5, 0x96, 0xB5, 0xB6, 0xB5, 0xD7, 0xBD,
        0xD7, 0xBD, 0xF3, 0xA4, 0xEF, 0x93, 0xCF, 0x93, 0x10, 0x9C, 0xDB, 0xE6,
  /* RLE: 040 Pixels @ 113,071*/ 40, 0xFF, 0xFF,
  /* ABS: 053 Pixels @ 025,072*/ 0, 53, 0xFB, 0xE6, 0xEF, 0x9B, 0xCF, 0x93, 0x75, 0xBD, 0xBA, 0xDE, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6,
        0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x3C, 0xEF, 0x18, 0xCE, 0x72, 0xA4, 0x72, 0xA4, 0x92, 0xA4, 0x92, 0xA4, 0xEF, 0x9B, 0xCF, 0x93, 0xEF, 0x9B, 0x71, 0xA4, 0xF3, 0xAC,
        0x55, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 015 Pixels @ 078,072*/ 15, 0xB2, 0x94,
  /* ABS: 021 Pixels @ 093,072*/ 0, 21, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0x14, 0xA5, 0x34, 0xA5, 0x34, 0xA5, 0x55, 0xAD, 0x92, 0x94, 0x8E, 0x73, 0x75, 0xAD, 0xB6, 0xB5, 0xB6, 0xB5, 0x14, 0xA5,
        0x10, 0x94, 0xCF, 0x93, 0xCF, 0x9B, 0x38, 0xD6,
  /* RLE: 040 Pixels @ 114,072*/ 40, 0xFF, 0xFF,
  /* ABS: 053 Pixels @ 026,073*/ 0, 53, 0x9A, 0xDE, 0xCF, 0x9B, 0xCF, 0x93, 0xB6, 0xC5, 0xBA, 0xDE, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE,
        0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x5D, 0xEF, 0x76, 0xBD, 0x92, 0xA4, 0x92, 0xA4, 0xB2, 0xA4, 0xB2, 0xA4, 0xEF, 0x9B, 0x51, 0xA4, 0xDF, 0xFF, 0x59, 0xCE, 0xF7, 0xBD, 0xD7, 0xBD,
        0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 017 Pixels @ 079,073*/ 17, 0xB2, 0x94,
  /* ABS: 020 Pixels @ 096,073*/ 0, 20, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0x30, 0x84, 0x0C, 0x63, 0xC7, 0x39, 0x82, 0x10, 0x00, 0x00, 0x00, 0x00, 0x65, 0x29, 0x14, 0xA5, 0x75, 0xAD, 0x96, 0xB5, 0x34, 0xAD, 0x30, 0x94, 0xCF, 0x93,
        0xCF, 0x93, 0x55, 0xBD, 0xDF, 0xFF,
  /* RLE: 039 Pixels @ 116,073*/ 39, 0xFF, 0xFF,
  /* ABS: 053 Pixels @ 027,074*/ 0, 53, 0x38, 0xD6, 0xCF, 0x93, 0xCF, 0x9B, 0xF7, 0xC5, 0x9A, 0xDE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE,
        0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x9E, 0xF7, 0xF3, 0xAC, 0xB2, 0xA4, 0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC, 0xEF, 0x9B, 0x71, 0xA4, 0xFF, 0xFF, 0xDF, 0xFF, 0x59, 0xCE, 0xF7, 0xBD, 0xF7, 0xBD,
        0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C,
  /* RLE: 018 Pixels @ 080,074*/ 18, 0xB2, 0x94,
  /* ABS: 003 Pixels @ 098,074*/ 0, 3, 0xD3, 0x9C, 0x0C, 0x63, 0x00, 0x00,
  /* RLE: 006 Pixels @ 101,074*/ 6, 0x00, 0x00,
  /* ABS: 010 Pixels @ 107,074*/ 0, 10, 0xC3, 0x18, 0x71, 0x8C, 0x55, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x71, 0x9C, 0xCF, 0x93, 0xCF, 0x93, 0xB2, 0xAC, 0x9E, 0xF7,
  /* RLE: 039 Pixels @ 117,074*/ 39, 0xFF, 0xFF,
  /* ABS: 054 Pixels @ 028,075*/ 0, 54, 0xD7, 0xCD, 0xCF, 0x93, 0xEF, 0x9B, 0x18, 0xCE, 0x9A, 0xDE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE,
        0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x7D, 0xF7, 0xB2, 0xA4, 0xB3, 0xA4, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xEF, 0x9B, 0x51, 0xA4, 0xFF, 0xFF, 0xFF, 0xFF, 0xDF, 0xFF, 0x79, 0xCE, 0xF7, 0xBD, 0xF7, 0xBD,
        0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 018 Pixels @ 082,075*/ 18, 0xB2, 0x94,
  /* ABS: 002 Pixels @ 100,075*/ 0, 2, 0x2C, 0x63, 0x20, 0x00,
  /* RLE: 006 Pixels @ 102,075*/ 6, 0x00, 0x00,
  /* ABS: 010 Pixels @ 108,075*/ 0, 10, 0x41, 0x08, 0xCF, 0x7B, 0x34, 0xA5, 0x55, 0xAD, 0x55, 0xAD, 0x92, 0x9C, 0xCF, 0x93, 0xCF, 0x93, 0x30, 0xA4, 0x3C, 0xEF,
  /* RLE: 039 Pixels @ 118,075*/ 39, 0xFF, 0xFF,
  /* ABS: 054 Pixels @ 029,076*/ 0, 54, 0x75, 0xBD, 0xCF, 0x93, 0x10, 0x9C, 0x38, 0xCE, 0x9A, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE,
        0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x3C, 0xEF, 0xD3, 0xAC, 0xD3, 0xAC, 0xD3, 0xAC, 0xF3, 0xAC, 0xF3, 0xAC, 0xCF, 0x9B, 0x71, 0xA4, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xDF, 0xFF, 0x79, 0xCE, 0xF7, 0xBD, 0xF7, 0xBD,
        0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 018 Pixels @ 083,076*/ 18, 0xB2, 0x94,
  /* ABS: 002 Pixels @ 101,076*/ 0, 2, 0xCF, 0x7B, 0x61, 0x08,
  /* RLE: 005 Pixels @ 103,076*/ 5, 0x00, 0x00,
  /* ABS: 011 Pixels @ 108,076*/ 0, 11, 0x20, 0x00, 0x65, 0x29, 0x92, 0x94, 0x14, 0xA5, 0x34, 0xA5, 0x34, 0xA5, 0xD3, 0x9C, 0xEF, 0x93, 0xCF, 0x93, 0xEF, 0x9B, 0x79, 0xDE,
  /* RLE: 038 Pixels @ 119,076*/ 38, 0xFF, 0xFF,
  /* ABS: 055 Pixels @ 029,077*/ 0, 55, 0xFF, 0xFF, 0x14, 0xB5, 0xCF, 0x93, 0x30, 0xA4, 0x59, 0xD6, 0x9A, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE,
        0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xFB, 0xE6, 0xF3, 0xAC, 0xF3, 0xAC, 0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xB5, 0xCF, 0x9B, 0x92, 0xAC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xDF, 0xFF, 0x79, 0xCE, 0x18, 0xC6,
        0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 018 Pixels @ 084,077*/ 18, 0xB2, 0x94,
  /* ABS: 019 Pixels @ 102,077*/ 0, 19, 0x30, 0x84, 0xC3, 0x18, 0x00, 0x00, 0xC3, 0x18, 0x08, 0x42, 0x6D, 0x6B, 0x92, 0x94, 0xD3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0x14, 0xA5, 0xD3, 0x9C, 0x10, 0x94, 0xCF, 0x93, 0xCF, 0x93,
        0xB6, 0xC5, 0xFF, 0xFF,
  /* RLE: 037 Pixels @ 121,077*/ 37, 0xFF, 0xFF,
  /* ABS: 029 Pixels @ 030,078*/ 0, 29, 0xDF, 0xFF, 0xD3, 0xAC, 0xCF, 0x93, 0x51, 0xA4, 0x59, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE,
        0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xDB, 0xE6, 0xF4, 0xAC, 0x14, 0xAD, 0x14, 0xB5, 0x14, 0xB5, 0x34, 0xB5, 0xCF, 0x9B, 0x92, 0xAC,
  /* RLE: 004 Pixels @ 059,078*/ 4, 0xFF, 0xFF,
  /* ABS: 022 Pixels @ 063,078*/ 0, 22, 0xFF, 0xFF, 0xFF, 0xFF, 0x59, 0xCE, 0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0xD3, 0x9C,
        0x92, 0x94, 0x14, 0xA5, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 018 Pixels @ 085,078*/ 18, 0xB2, 0x94,
  /* ABS: 002 Pixels @ 103,078*/ 0, 2, 0x71, 0x8C, 0x51, 0x8C,
  /* RLE: 004 Pixels @ 105,078*/ 4, 0xB2, 0x94,
  /* ABS: 013 Pixels @ 109,078*/ 0, 13, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xF3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0xF3, 0x9C, 0x30, 0x94, 0xCF, 0x93, 0xCF, 0x93, 0x14, 0xB5, 0xBE, 0xFF,
  /* RLE: 037 Pixels @ 122,078*/ 37, 0xFF, 0xFF,
  /* ABS: 056 Pixels @ 031,079*/ 0, 56, 0xBE, 0xFF, 0x92, 0xAC, 0xCF, 0x93, 0x92, 0xA4, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE,
        0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xBA, 0xDE, 0x14, 0xB5, 0x14, 0xB5, 0x34, 0xB5, 0x34, 0xB5, 0x35, 0xB5, 0xCF, 0x9B, 0xB2, 0xAC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x59, 0xCE, 0x18, 0xC6,
        0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x55, 0xAD, 0x10, 0x84, 0xAA, 0x52, 0x24, 0x21, 0x00, 0x00, 0x61, 0x08, 0x10, 0x84, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 026 Pixels @ 087,079*/ 26, 0xB2, 0x94,
  /* ABS: 010 Pixels @ 113,079*/ 0, 10, 0xD3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0x71, 0x94, 0xCF, 0x93, 0xCF, 0x93, 0x71, 0xA4, 0x5D, 0xF7,
  /* RLE: 037 Pixels @ 123,079*/ 37, 0xFF, 0xFF,
  /* ABS: 044 Pixels @ 032,080*/ 0, 44, 0x9E, 0xF7, 0x51, 0xA4, 0xCF, 0x93, 0x71, 0xA4, 0x59, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE,
        0x18, 0xCE, 0xF8, 0xCD, 0x7A, 0xD6, 0x34, 0xB5, 0x34, 0xB5, 0x35, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0xCF, 0x9B, 0xB2, 0xAC, 0xBE, 0xF7, 0xDF, 0xFF, 0xDF, 0xFF, 0xDF, 0xFF, 0xDF, 0xFF, 0xDF, 0xFF, 0xFF, 0xFF, 0xDF, 0xFF, 0x38, 0xC6, 0x18, 0xC6,
        0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0x96, 0xB5, 0x2C, 0x63, 0x86, 0x31, 0x20, 0x00,
  /* RLE: 005 Pixels @ 076,080*/ 5, 0x00, 0x00,
  /* ABS: 007 Pixels @ 081,080*/ 0, 7, 0x20, 0x00, 0xCF, 0x7B, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 031 Pixels @ 088,080*/ 31, 0xB2, 0x94,
  /* ABS: 005 Pixels @ 119,080*/ 0, 5, 0x92, 0x94, 0xEF, 0x93, 0xCF, 0x93, 0x10, 0x9C, 0xDB, 0xE6,
  /* RLE: 037 Pixels @ 124,080*/ 37, 0xFF, 0xFF,
  /* ABS: 041 Pixels @ 033,081*/ 0, 41, 0x5D, 0xEF, 0x30, 0xA4, 0xCF, 0x93, 0x71, 0xA4, 0x59, 0xCE, 0x79, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE,
        0xF8, 0xCD, 0x59, 0xD6, 0x55, 0xB5, 0x55, 0xB5, 0x55, 0xB5, 0x75, 0xBD, 0x75, 0xBD, 0xCF, 0x93, 0xD3, 0xAC, 0x9E, 0xF7, 0xBE, 0xF7, 0xBE, 0xF7, 0xBE, 0xF7, 0xBE, 0xF7, 0xBE, 0xF7, 0xDF, 0xFF, 0xFF, 0xFF, 0x7D, 0xEF, 0x38, 0xC6, 0x18, 0xC6,
        0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0x2C, 0x63,
  /* RLE: 008 Pixels @ 074,081*/ 8, 0x00, 0x00,
  /* ABS: 007 Pixels @ 082,081*/ 0, 7, 0x20, 0x00, 0x6D, 0x6B, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 031 Pixels @ 089,081*/ 31, 0xB2, 0x94,
  /* ABS: 005 Pixels @ 120,081*/ 0, 5, 0xB2, 0x94, 0x10, 0x94, 0xCF, 0x93, 0xCF, 0x9B, 0x18, 0xD6,
  /* RLE: 037 Pixels @ 125,081*/ 37, 0xFF, 0xFF,
  /* ABS: 042 Pixels @ 034,082*/ 0, 42, 0x7D, 0xF7, 0x51, 0xA4, 0xCF, 0x93, 0x51, 0xA4, 0x38, 0xCE, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD,
        0x38, 0xCE, 0x55, 0xBD, 0x75, 0xBD, 0x75, 0xBD, 0x76, 0xBD, 0x96, 0xBD, 0xCF, 0x93, 0xD3, 0xAC, 0x7D, 0xEF, 0x9E, 0xF7, 0x9E, 0xF7, 0x9E, 0xF7, 0x9E, 0xF7, 0x9E, 0xF7, 0x9E, 0xF7, 0xBE, 0xF7, 0xFF, 0xFF, 0x1C, 0xE7, 0x18, 0xC6, 0x18, 0xC6,
        0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0x6D, 0x6B, 0x00, 0x00,
  /* RLE: 007 Pixels @ 076,082*/ 7, 0x00, 0x00,
  /* ABS: 007 Pixels @ 083,082*/ 0, 7, 0x00, 0x00, 0x0C, 0x63, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 031 Pixels @ 090,082*/ 31, 0xB2, 0x94,
  /* ABS: 006 Pixels @ 121,082*/ 0, 6, 0xB2, 0x94, 0x30, 0x94, 0xCF, 0x93, 0xCF, 0x93, 0x55, 0xBD, 0xDF, 0xFF,
  /* RLE: 036 Pixels @ 127,082*/ 36, 0xFF, 0xFF,
  /* ABS: 042 Pixels @ 035,083*/ 0, 42, 0x7D, 0xF7, 0x51, 0xA4, 0xCF, 0x93, 0x51, 0xA4, 0x38, 0xCE, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x18, 0xCE,
        0x75, 0xBD, 0x76, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0x96, 0xBD, 0xCF, 0x93, 0x92, 0xAC, 0x5D, 0xEF, 0x5D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x7D, 0xEF, 0x9E, 0xF7, 0xFF, 0xFF, 0xBA, 0xD6, 0xF7, 0xBD, 0x18, 0xC6,
        0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xAE, 0x73, 0x00, 0x00,
  /* RLE: 004 Pixels @ 077,083*/ 4, 0x00, 0x00,
  /* ABS: 011 Pixels @ 081,083*/ 0, 11, 0x00, 0x00, 0x04, 0x21, 0x69, 0x4A, 0xCF, 0x7B, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 031 Pixels @ 092,083*/ 31, 0xB2, 0x94,
  /* ABS: 005 Pixels @ 123,083*/ 0, 5, 0x51, 0x94, 0xCF, 0x93, 0xCF, 0x93, 0xB2, 0xAC, 0xBE, 0xFF,
  /* RLE: 036 Pixels @ 000,084*/ 36, 0xFF, 0xFF,
  /* ABS: 057 Pixels @ 036,084*/ 0, 57, 0x7D, 0xF7, 0x51, 0xA4, 0xCF, 0x93, 0x30, 0xA4, 0x18, 0xCE, 0x59, 0xD6, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xF7, 0xC5, 0x96, 0xBD,
        0x96, 0xBD, 0x96, 0xBD, 0xB6, 0xC5, 0xB6, 0xC5, 0xEF, 0x9B, 0xCF, 0x93, 0xB6, 0xC5, 0x3C, 0xE7, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x5D, 0xEF, 0x9E, 0xF7, 0x9E, 0xF7, 0x59, 0xCE, 0xD7, 0xBD, 0x18, 0xC6,
        0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xEF, 0x7B, 0x20, 0x00, 0xE3, 0x18, 0x69, 0x4A, 0xCF, 0x7B, 0x34, 0xA5, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 030 Pixels @ 093,084*/ 30, 0xB2, 0x94,
  /* ABS: 005 Pixels @ 123,084*/ 0, 5, 0x14, 0xA5, 0xB6, 0xBD, 0xEF, 0x93, 0xCF, 0x93, 0x3C, 0xEF,
  /* RLE: 037 Pixels @ 000,085*/ 37, 0xFF, 0xFF,
  /* ABS: 057 Pixels @ 037,085*/ 0, 57, 0x9E, 0xF7, 0x71, 0xA4, 0xCF, 0x93, 0x30, 0xA4, 0x18, 0xCE, 0x59, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xD7, 0xC5, 0xB6, 0xBD, 0xB6, 0xC5,
        0xB7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0x55, 0xBD, 0xCF, 0x9B, 0xCF, 0x93, 0xD7, 0xC5, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x3C, 0xE7, 0x7D, 0xEF, 0x1C, 0xE7, 0xF7, 0xBD, 0xD7, 0xBD, 0x38, 0xC6,
        0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xB6, 0xB5, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 025 Pixels @ 094,085*/ 25, 0xB2, 0x94,
  /* ABS: 009 Pixels @ 119,085*/ 0, 9, 0xD3, 0x9C, 0x96, 0xB5, 0x79, 0xCE, 0x3C, 0xE7, 0xFF, 0xFF, 0xFF, 0xFF, 0xCF, 0x83, 0xCF, 0x93, 0x5D, 0xF7,
  /* RLE: 038 Pixels @ 000,086*/ 38, 0xFF, 0xFF,
  /* ABS: 057 Pixels @ 038,086*/ 0, 57, 0x9E, 0xF7, 0x71, 0xAC, 0xCF, 0x93, 0x30, 0x9C, 0xF7, 0xCD, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xD7, 0xC5, 0xB7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5,
        0xD7, 0xC5, 0xF7, 0xC5, 0xF7, 0xC5, 0x75, 0xBD, 0xCF, 0x9B, 0xCF, 0x93, 0x18, 0xCE, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x1C, 0xE7, 0x7D, 0xEF, 0xBA, 0xD6, 0x96, 0xB5, 0xD7, 0xBD, 0x38, 0xC6,
        0x18, 0xC6, 0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xD3, 0x9C, 0xD3, 0x9C,
  /* RLE: 021 Pixels @ 095,086*/ 21, 0xB2, 0x94,
  /* ABS: 012 Pixels @ 116,086*/ 0, 12, 0x34, 0xA5, 0x18, 0xC6, 0xBA, 0xD6, 0xBE, 0xF7, 0xFF, 0xFF, 0x5D, 0xEF, 0x14, 0xA5, 0xCB, 0x5A, 0xA2, 0x10, 0xCB, 0x62, 0xCF, 0x93, 0x9E, 0xF7,
  /* RLE: 039 Pixels @ 000,087*/ 39, 0xFF, 0xFF,
  /* ABS: 057 Pixels @ 039,087*/ 0, 57, 0xBE, 0xF7, 0x92, 0xAC, 0xCF, 0x93, 0x10, 0x9C, 0xF7, 0xC5, 0x38, 0xCE, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xD7, 0xC5, 0xD7, 0xC5, 0xD7, 0xC5, 0xF7, 0xC5, 0xF7, 0xC5,
        0xF8, 0xCD, 0x18, 0xCE, 0x18, 0xCE, 0x55, 0xBD, 0xCF, 0x93, 0xCF, 0x93, 0x59, 0xCE, 0xFB, 0xDE, 0xFB, 0xDE, 0xFB, 0xDE, 0xFB, 0xDE, 0xFB, 0xDE, 0xFB, 0xDE, 0xFB, 0xDE, 0xFB, 0xDE, 0x3C, 0xE7, 0x59, 0xCE, 0x14, 0xA5, 0xB6, 0xB5, 0x38, 0xC6,
        0x18, 0xC6, 0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C,
  /* RLE: 016 Pixels @ 096,087*/ 16, 0xB2, 0x94,
  /* ABS: 016 Pixels @ 112,087*/ 0, 16, 0xD3, 0x9C, 0xB6, 0xB5, 0x79, 0xCE, 0x5D, 0xEF, 0xFF, 0xFF, 0xDF, 0xFF, 0x18, 0xC6, 0xCF, 0x7B, 0x86, 0x31, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xEB, 0x6A, 0xCF, 0x93, 0xBE, 0xF7,
  /* RLE: 040 Pixels @ 000,088*/ 40, 0xFF, 0xFF,
  /* ABS: 058 Pixels @ 040,088*/ 0, 58, 0xBE, 0xFF, 0x92, 0xAC, 0xCF, 0x93, 0x10, 0x9C, 0xD7, 0xC5, 0x38, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xF7, 0xC5, 0xF7, 0xC5, 0xF7, 0xCD, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE,
        0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x55, 0xBD, 0xCF, 0x93, 0xCF, 0x93, 0x79, 0xD6, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xDB, 0xDE, 0xFB, 0xDE, 0xF7, 0xBD, 0xB2, 0x94, 0xB6, 0xB5, 0x38, 0xC6,
        0x18, 0xC6, 0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C,
        0xD3, 0x9C,
  /* RLE: 011 Pixels @ 098,088*/ 11, 0xB2, 0x94,
  /* ABS: 009 Pixels @ 109,088*/ 0, 9, 0x55, 0xAD, 0x38, 0xC6, 0xFB, 0xDE, 0xDF, 0xFF, 0xFF, 0xFF, 0x1C, 0xE7, 0xD3, 0x9C, 0x8A, 0x52, 0x61, 0x08,
  /* RLE: 007 Pixels @ 118,088*/ 7, 0x00, 0x00,
  /* ABS: 003 Pixels @ 125,088*/ 0, 3, 0xEB, 0x72, 0xCF, 0x93, 0xDF, 0xFF,
  /* RLE: 041 Pixels @ 000,089*/ 41, 0xFF, 0xFF,
  /* ABS: 058 Pixels @ 041,089*/ 0, 58, 0xBE, 0xFF, 0xB2, 0xAC, 0xCF, 0x93, 0x10, 0x9C, 0xB7, 0xC5, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0xF7, 0xCD, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x38, 0xCE,
        0x59, 0xCE, 0x59, 0xD6, 0x59, 0xD6, 0x34, 0xB5, 0xCF, 0x93, 0xCF, 0x93, 0x79, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0xBA, 0xD6, 0x75, 0xAD, 0xD3, 0x9C, 0xB6, 0xB5, 0x38, 0xC6,
        0x18, 0xC6, 0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C,
        0xD3, 0x9C,
  /* RLE: 006 Pixels @ 099,089*/ 6, 0xB2, 0x94,
  /* ABS: 009 Pixels @ 105,089*/ 0, 9, 0xF3, 0x9C, 0xD7, 0xBD, 0x9A, 0xD6, 0x7D, 0xEF, 0xFF, 0xFF, 0xDF, 0xFF, 0xD7, 0xBD, 0x8E, 0x73, 0x45, 0x29,
  /* RLE: 011 Pixels @ 114,089*/ 11, 0x00, 0x00,
  /* ABS: 003 Pixels @ 125,089*/ 0, 3, 0x0C, 0x73, 0xCF, 0x93, 0xFF, 0xFF,
  /* RLE: 042 Pixels @ 000,090*/ 42, 0xFF, 0xFF,
  /* ABS: 069 Pixels @ 042,090*/ 0, 69, 0xDF, 0xFF, 0xD3, 0xAC, 0xCF, 0x93, 0xEF, 0x9B, 0xB6, 0xC5, 0x18, 0xCE, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x18, 0xCE, 0x18, 0xCE, 0x38, 0xCE, 0x38, 0xCE, 0x39, 0xCE, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6,
        0x79, 0xD6, 0x79, 0xD6, 0x7A, 0xD6, 0xF4, 0xB4, 0xCF, 0x93, 0xEF, 0x9B, 0x79, 0xCE, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x14, 0xA5, 0xB2, 0x94, 0xB6, 0xB5, 0x38, 0xC6,
        0x38, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0xF3, 0x9C, 0xF3, 0x9C, 0xD3, 0x9C,
        0xD3, 0x9C, 0xB2, 0x94, 0xB2, 0x94, 0x75, 0xAD, 0x59, 0xCE, 0x1C, 0xE7, 0xFF, 0xFF, 0xFF, 0xFF, 0xDB, 0xDE, 0x92, 0x94, 0x49, 0x4A, 0x41, 0x08,
  /* RLE: 014 Pixels @ 111,090*/ 14, 0x00, 0x00,
  /* ABS: 002 Pixels @ 125,090*/ 0, 2, 0x2C, 0x7B, 0xCF, 0x9B,
  /* RLE: 044 Pixels @ 127,090*/ 44, 0xFF, 0xFF,
  /* ABS: 064 Pixels @ 043,091*/ 0, 64, 0xDF, 0xFF, 0xD3, 0xB4, 0xCF, 0x93, 0xEF, 0x9B, 0x96, 0xBD, 0x18, 0xCE, 0x18, 0xCE, 0xF8, 0xCD, 0x18, 0xCE, 0x38, 0xCE, 0x39, 0xCE, 0x59, 0xD6, 0x59, 0xD6, 0x59, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x7A, 0xD6,
        0x9A, 0xD6, 0x9A, 0xDE, 0x9A, 0xDE, 0xD3, 0xAC, 0xCF, 0x93, 0x10, 0x9C, 0x59, 0xCE, 0x79, 0xCE, 0x79, 0xCE, 0x79, 0xCE, 0x79, 0xCE, 0x79, 0xCE, 0x79, 0xCE, 0x79, 0xCE, 0x79, 0xCE, 0x59, 0xCE, 0xB2, 0x94, 0xB2, 0x94, 0xB6, 0xB5, 0x38, 0xC6,
        0x38, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x34, 0xA5, 0x34, 0xA5, 0x14, 0xA5, 0x14, 0xA5, 0x34, 0xA5, 0x18, 0xC6,
        0xDB, 0xDE, 0x9E, 0xF7, 0xFF, 0xFF, 0x9E, 0xF7, 0x96, 0xB5, 0x4D, 0x6B, 0x04, 0x21,
  /* RLE: 018 Pixels @ 107,091*/ 18, 0x00, 0x00,
  /* ABS: 002 Pixels @ 125,091*/ 0, 2, 0x4D, 0x7B, 0x10, 0x9C,
  /* RLE: 045 Pixels @ 127,091*/ 45, 0xFF, 0xFF,
  /* ABS: 060 Pixels @ 044,092*/ 0, 60, 0xDF, 0xFF, 0xF3, 0xB4, 0xCF, 0x93, 0xCF, 0x9B, 0x34, 0xB5, 0x18, 0xCE, 0xF8, 0xCD, 0x38, 0xCE, 0x59, 0xD6, 0x59, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xDE, 0xBA, 0xDE,
        0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xB2, 0xAC, 0xCF, 0x93, 0x30, 0xA4, 0x59, 0xCE, 0x59, 0xCE, 0x59, 0xCE, 0x59, 0xCE, 0x59, 0xCE, 0x59, 0xCE, 0x59, 0xCE, 0x59, 0xCE, 0x79, 0xCE, 0x18, 0xC6, 0xB2, 0x94, 0xB2, 0x94, 0xB6, 0xB5, 0x38, 0xC6,
        0x38, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0x96, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0xF7, 0xBD, 0xBA, 0xD6, 0x3C, 0xE7, 0xFF, 0xFF, 0xFF, 0xFF, 0x9A, 0xD6,
        0x51, 0x8C, 0x08, 0x42, 0x20, 0x00,
  /* RLE: 021 Pixels @ 104,092*/ 21, 0x00, 0x00,
  /* ABS: 002 Pixels @ 125,092*/ 0, 2, 0x6D, 0x83, 0x30, 0xA4,
  /* RLE: 046 Pixels @ 127,092*/ 46, 0xFF, 0xFF,
  /* ABS: 055 Pixels @ 045,093*/ 0, 55, 0xDF, 0xFF, 0x34, 0xBD, 0xCF, 0x93, 0xCF, 0x93, 0xB3, 0xAC, 0xF7, 0xC5, 0x39, 0xCE, 0x79, 0xD6, 0x79, 0xD6, 0x79, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0xDB, 0xDE,
        0xBA, 0xDE, 0x18, 0xCE, 0x34, 0xB5, 0xEF, 0x9B, 0xCF, 0x93, 0x71, 0xA4, 0x38, 0xC6, 0x38, 0xC6, 0x38, 0xC6, 0x38, 0xC6, 0x38, 0xC6, 0x38, 0xC6, 0x38, 0xC6, 0x38, 0xC6, 0x59, 0xCE, 0xD7, 0xBD, 0xB2, 0x94, 0xB2, 0x94, 0xB6, 0xB5, 0x59, 0xCE,
        0x38, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xB6, 0xB5, 0xB6, 0xB5, 0xF7, 0xBD, 0x9A, 0xD6, 0x1C, 0xE7, 0xDF, 0xFF, 0xFF, 0xFF, 0x7D, 0xEF, 0x55, 0xAD, 0x0C, 0x63, 0xC3, 0x18,
  /* RLE: 025 Pixels @ 100,093*/ 25, 0x00, 0x00,
  /* ABS: 002 Pixels @ 125,093*/ 0, 2, 0x6D, 0x8B, 0x51, 0xA4,
  /* RLE: 047 Pixels @ 127,093*/ 47, 0xFF, 0xFF,
  /* ABS: 017 Pixels @ 046,094*/ 0, 17, 0xFF, 0xFF, 0xF7, 0xCD, 0xCF, 0x93, 0xCF, 0x93, 0x51, 0xA4, 0xF7, 0xCD, 0x7A, 0xD6, 0x9A, 0xD6, 0x9A, 0xD6, 0x9A, 0xDE, 0xBA, 0xDE, 0xBA, 0xDE, 0x79, 0xD6, 0xB6, 0xC5, 0xF3, 0xB4, 0x51, 0xA4, 0xCF, 0x9B,
  /* RLE: 005 Pixels @ 063,094*/ 5, 0xCF, 0x93,
  /* ABS: 029 Pixels @ 068,094*/ 0, 29, 0x92, 0xAC, 0x18, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0x18, 0xC6, 0x38, 0xC6, 0x38, 0xC6, 0x75, 0xAD, 0xB2, 0x94, 0xB2, 0x94, 0xB6, 0xB5, 0x59, 0xCE, 0x38, 0xC6, 0x38, 0xC6,
        0x18, 0xC6, 0xF7, 0xBD, 0xF7, 0xBD, 0x79, 0xCE, 0x1C, 0xE7, 0x7D, 0xEF, 0xFF, 0xFF, 0xFF, 0xFF, 0x38, 0xC6, 0xEF, 0x7B, 0xC7, 0x39, 0x00, 0x00,
  /* RLE: 028 Pixels @ 097,094*/ 28, 0x00, 0x00,
  /* ABS: 002 Pixels @ 125,094*/ 0, 2, 0x8E, 0x8B, 0x71, 0xA4,
  /* RLE: 049 Pixels @ 127,094*/ 49, 0xFF, 0xFF,
  /* ABS: 010 Pixels @ 048,095*/ 0, 10, 0xBA, 0xDE, 0xEF, 0x9B, 0xCF, 0x93, 0x10, 0x9C, 0x96, 0xC5, 0x9A, 0xDE, 0x18, 0xCE, 0x55, 0xBD, 0xB2, 0xAC, 0x10, 0x9C,
  /* RLE: 005 Pixels @ 058,095*/ 5, 0xCF, 0x93,
  /* ABS: 007 Pixels @ 063,095*/ 0, 7, 0xCF, 0x9B, 0xB2, 0xAC, 0x96, 0xC5, 0x59, 0xD6, 0x51, 0xA4, 0xCF, 0x93, 0xD3, 0xAC,
  /* RLE: 004 Pixels @ 070,095*/ 4, 0xF7, 0xBD,
  /* RLE: 003 Pixels @ 074,095*/ 3, 0xF7, 0xBD,
  /* ABS: 016 Pixels @ 077,095*/ 0, 16, 0x18, 0xC6, 0x18, 0xC6, 0x75, 0xAD, 0xB2, 0x94, 0xB2, 0x94, 0xB6, 0xB5, 0x59, 0xCE, 0x79, 0xCE, 0xFB, 0xDE, 0x5D, 0xEF, 0xDF, 0xFF, 0xFF, 0xFF, 0x3C, 0xE7, 0xF3, 0x9C, 0xAA, 0x52, 0x82, 0x10,
  /* RLE: 032 Pixels @ 093,095*/ 32, 0x00, 0x00,
  /* ABS: 002 Pixels @ 125,095*/ 0, 2, 0xAE, 0x93, 0x92, 0xAC,
  /* RLE: 050 Pixels @ 127,095*/ 50, 0xFF, 0xFF,
  /* ABS: 005 Pixels @ 049,096*/ 0, 5, 0x3C, 0xEF, 0x51, 0xA4, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x9B,
  /* RLE: 005 Pixels @ 054,096*/ 5, 0xCF, 0x93,
  /* ABS: 030 Pixels @ 059,096*/ 0, 30, 0x10, 0x9C, 0xF3, 0xB4, 0xF7, 0xCD, 0xFB, 0xE6, 0xDF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7D, 0xF7, 0x30, 0xA4, 0xCF, 0x93, 0x14, 0xB5, 0xD7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD,
        0xD7, 0xBD, 0xF7, 0xBD, 0xF7, 0xBD, 0xF7, 0xBD, 0x55, 0xAD, 0xB2, 0x94, 0x96, 0xB5, 0x7D, 0xEF, 0xFF, 0xFF, 0xDF, 0xFF, 0xF7, 0xBD, 0xAE, 0x73, 0x65, 0x29,
  /* RLE: 033 Pixels @ 089,096*/ 33, 0x00, 0x00,
  /* ABS: 005 Pixels @ 122,096*/ 0, 5, 0x00, 0x00, 0x04, 0x21, 0x28, 0x52, 0xCF, 0x93, 0xB2, 0xAC,
  /* RLE: 051 Pixels @ 127,096*/ 51, 0xFF, 0xFF,
  /* ABS: 010 Pixels @ 050,097*/ 0, 10, 0xBE, 0xFF, 0x96, 0xC5, 0xCF, 0x9B, 0xCF, 0x93, 0xCF, 0x93, 0x51, 0xA4, 0x55, 0xBD, 0x38, 0xD6, 0x3C, 0xEF, 0xFF, 0xFF,
  /* RLE: 008 Pixels @ 060,097*/ 8, 0xFF, 0xFF,
  /* ABS: 018 Pixels @ 068,097*/ 0, 18, 0x5D, 0xEF, 0x10, 0x9C, 0xCF, 0x93, 0x34, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0xD7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0xD7, 0xBD, 0x55, 0xAD, 0xBE, 0xF7, 0x96, 0xB5, 0x69, 0x4A,
        0x61, 0x08,
  /* RLE: 033 Pixels @ 086,097*/ 33, 0x00, 0x00,
  /* ABS: 008 Pixels @ 119,097*/ 0, 8, 0x41, 0x08, 0x86, 0x31, 0x8A, 0x62, 0x8E, 0x8B, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xD3, 0xB4,
  /* RLE: 053 Pixels @ 127,097*/ 53, 0xFF, 0xFF,
  /* ABS: 003 Pixels @ 052,098*/ 0, 3, 0x5D, 0xEF, 0xBA, 0xDE, 0x9E, 0xF7,
  /* RLE: 014 Pixels @ 055,098*/ 14, 0xFF, 0xFF,
  /* ABS: 015 Pixels @ 069,098*/ 0, 15, 0x1C, 0xEF, 0xEF, 0x9B, 0xCF, 0x93, 0x55, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0xB6, 0xB5, 0xF7, 0xBD, 0xFF, 0xFF, 0x00, 0x00,
  /* RLE: 032 Pixels @ 084,098*/ 32, 0x00, 0x00,
  /* ABS: 011 Pixels @ 116,098*/ 0, 11, 0xC3, 0x18, 0xE7, 0x41, 0xEB, 0x72, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0x10, 0x9C, 0x55, 0xBD, 0x9A, 0xDE, 0xBE, 0xF7,
  /* RLE: 071 Pixels @ 127,098*/ 71, 0xFF, 0xFF,
  /* ABS: 013 Pixels @ 070,099*/ 0, 13, 0xDB, 0xE6, 0xEF, 0x9B, 0xCF, 0x93, 0x75, 0xB5, 0x75, 0xAD, 0x75, 0xAD, 0x75, 0xAD, 0x75, 0xAD, 0x96, 0xB5, 0x96, 0xB5, 0x96, 0xB5, 0xD7, 0xBD, 0xFF, 0xFF,
  /* RLE: 029 Pixels @ 083,099*/ 29, 0x00, 0x00,
  /* ABS: 004 Pixels @ 112,099*/ 0, 4, 0x20, 0x00, 0x45, 0x29, 0x69, 0x5A, 0x6D, 0x83,
  /* RLE: 004 Pixels @ 116,099*/ 4, 0xCF, 0x93,
  /* ABS: 004 Pixels @ 120,099*/ 0, 4, 0x71, 0xAC, 0x38, 0xD6, 0x3C, 0xEF, 0xFF, 0xFF,
  /* RLE: 075 Pixels @ 124,099*/ 75, 0xFF, 0xFF,
  /* ABS: 012 Pixels @ 071,100*/ 0, 12, 0x9A, 0xDE, 0xCF, 0x9B, 0xCF, 0x93, 0x75, 0xB5, 0x55, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x55, 0xAD, 0x75, 0xAD, 0x75, 0xAD, 0xD7, 0xBD, 0xFF, 0xFF,
  /* RLE: 026 Pixels @ 083,100*/ 26, 0x00, 0x00,
  /* ABS: 012 Pixels @ 109,100*/ 0, 12, 0x82, 0x10, 0xC7, 0x39, 0xCB, 0x6A, 0xAE, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xEF, 0x9B, 0xF3, 0xB4, 0x79, 0xD6, 0xBE, 0xF7, 0xBE, 0xEF,
  /* RLE: 079 Pixels @ 121,100*/ 79, 0xFF, 0xFF,
  /* ABS: 011 Pixels @ 072,101*/ 0, 11, 0x59, 0xD6, 0xCF, 0x93, 0xCF, 0x93, 0x75, 0xB5, 0x34, 0xA5, 0x34, 0xA5, 0x34, 0xA5, 0x55, 0xAD, 0x55, 0xAD, 0xB6, 0xB5, 0xDF, 0xFF,
  /* RLE: 022 Pixels @ 083,101*/ 22, 0x00, 0x00,
  /* ABS: 004 Pixels @ 105,101*/ 0, 4, 0x00, 0x00, 0x04, 0x21, 0x28, 0x52, 0x2C, 0x7B,
  /* RLE: 004 Pixels @ 109,101*/ 4, 0xCF, 0x93,
  /* ABS: 006 Pixels @ 113,101*/ 0, 6, 0x30, 0xA4, 0x75, 0xC5, 0xBA, 0xDE, 0xDF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  /* RLE: 082 Pixels @ 119,101*/ 82, 0xFF, 0xFF,
  /* ABS: 010 Pixels @ 073,102*/ 0, 10, 0xF7, 0xCD, 0xCF, 0x93, 0xCF, 0x93, 0x75, 0xAD, 0x14, 0xA5, 0x14, 0xA5, 0x34, 0xA5, 0x34, 0xA5, 0x96, 0xB5, 0xDF, 0xFF,
  /* RLE: 019 Pixels @ 083,102*/ 19, 0x00, 0x00,
  /* ABS: 012 Pixels @ 102,102*/ 0, 12, 0x41, 0x08, 0xA6, 0x39, 0xAA, 0x62, 0x8E, 0x8B, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xB2, 0xAC, 0xF7, 0xCD, 0x3C, 0xEF, 0xFF, 0xFF,
  /* RLE: 088 Pixels @ 114,102*/ 88, 0xFF, 0xFF,
  /* ABS: 009 Pixels @ 074,103*/ 0, 9, 0xB6, 0xC5, 0xCF, 0x93, 0xEF, 0x9B, 0x55, 0xAD, 0xF3, 0x9C, 0xF3, 0x9C, 0x14, 0xA5, 0x75, 0xAD, 0xDF, 0xFF,
  /* RLE: 016 Pixels @ 083,103*/ 16, 0x00, 0x00,
  /* ABS: 011 Pixels @ 099,103*/ 0, 11, 0xE3, 0x18, 0x28, 0x4A, 0x0C, 0x73, 0xAE, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xEF, 0x9B, 0x34, 0xBD, 0x79, 0xD6, 0x9E, 0xF7,
  /* RLE: 092 Pixels @ 110,103*/ 92, 0xFF, 0xFF,
  /* ABS: 009 Pixels @ 074,104*/ 0, 9, 0xFF, 0xFF, 0x55, 0xBD, 0xCF, 0x93, 0x10, 0x9C, 0x34, 0xA5, 0xD3, 0x9C, 0xF3, 0x9C, 0x55, 0xAD, 0xBE, 0xF7,
  /* RLE: 012 Pixels @ 083,104*/ 12, 0x00, 0x00,
  /* ABS: 004 Pixels @ 095,104*/ 0, 4, 0x20, 0x00, 0x65, 0x31, 0x8A, 0x5A, 0x6D, 0x83,
  /* RLE: 004 Pixels @ 099,104*/ 4, 0xCF, 0x93,
  /* ABS: 004 Pixels @ 103,104*/ 0, 4, 0x51, 0xA4, 0x96, 0xC5, 0xDB, 0xE6, 0xFF, 0xFF,
  /* RLE: 096 Pixels @ 107,104*/ 96, 0xFF, 0xFF,
  /* ABS: 008 Pixels @ 075,105*/ 0, 8, 0xFF, 0xFF, 0x14, 0xB5, 0xCF, 0x93, 0x10, 0x9C, 0xF3, 0x9C, 0xB2, 0x94, 0x55, 0xAD, 0xBE, 0xF7,
  /* RLE: 009 Pixels @ 083,105*/ 9, 0x00, 0x00,
  /* ABS: 011 Pixels @ 092,105*/ 0, 11, 0xA2, 0x10, 0x08, 0x42, 0x0C, 0x73, 0x8E, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x9B, 0xD3, 0xB4, 0x18, 0xD6, 0x5D, 0xEF,
  /* RLE: 101 Pixels @ 103,105*/ 101, 0xFF, 0xFF,
  /* ABS: 007 Pixels @ 076,106*/ 0, 7, 0xDF, 0xFF, 0xD3, 0xB4, 0xCF, 0x93, 0x51, 0x9C, 0xF3, 0x9C, 0x55, 0xAD, 0xBE, 0xF7,
  /* RLE: 005 Pixels @ 083,106*/ 5, 0x00, 0x00,
  /* ABS: 004 Pixels @ 088,106*/ 0, 4, 0x00, 0x00, 0x45, 0x29, 0x8A, 0x5A, 0x4D, 0x83,
  /* RLE: 004 Pixels @ 092,106*/ 4, 0xCF, 0x93,
  /* ABS: 004 Pixels @ 096,106*/ 0, 4, 0x10, 0x9C, 0x55, 0xBD, 0x9A, 0xDE, 0xBE, 0xFF,
  /* RLE: 105 Pixels @ 100,106*/ 105, 0xFF, 0xFF,
  /* ABS: 012 Pixels @ 077,107*/ 0, 12, 0xDF, 0xFF, 0x92, 0xAC, 0xCF, 0x93, 0x71, 0xA4, 0x75, 0xAD, 0x9E, 0xF7, 0x00, 0x00, 0x00, 0x00, 0x61, 0x08, 0xE7, 0x41, 0xEB, 0x6A, 0x8E, 0x8B,
  /* RLE: 004 Pixels @ 089,107*/ 4, 0xCF, 0x93,
  /* ABS: 004 Pixels @ 093,107*/ 0, 4, 0x92, 0xAC, 0xD7, 0xCD, 0x1C, 0xE7, 0xFF, 0xFF,
  /* RLE: 109 Pixels @ 097,107*/ 109, 0xFF, 0xFF,
  /* ABS: 015 Pixels @ 078,108*/ 0, 15, 0x9E, 0xF7, 0x71, 0xA4, 0xCF, 0x93, 0xD3, 0xAC, 0x59, 0xCE, 0x69, 0x52, 0x4D, 0x7B, 0xAE, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xCF, 0x93, 0xEF, 0x9B, 0x14, 0xB5, 0x59, 0xD6, 0x7D, 0xF7,
  /* RLE: 114 Pixels @ 093,108*/ 114, 0xFF, 0xFF,
  /* ABS: 002 Pixels @ 079,109*/ 0, 2, 0x7D, 0xF7, 0x30, 0xA4,
  /* RLE: 005 Pixels @ 081,109*/ 5, 0xCF, 0x93,
  /* ABS: 004 Pixels @ 086,109*/ 0, 4, 0x51, 0xA4, 0x75, 0xC5, 0xBA, 0xE6, 0xDF, 0xFF,
  /* RLE: 118 Pixels @ 090,109*/ 118, 0xFF, 0xFF,
  /* ABS: 006 Pixels @ 080,110*/ 0, 6, 0x5D, 0xEF, 0x10, 0x9C, 0xCF, 0x9B, 0xB2, 0xAC, 0xF7, 0xCD, 0x3C, 0xEF,
  /* RLE: 123 Pixels @ 086,110*/ 123, 0xFF, 0xFF,
  /* ABS: 002 Pixels @ 081,111*/ 0, 2, 0x7D, 0xF7, 0xBE, 0xF7,
  /* RLE: 152 Pixels @ 083,111*/ 152, 0xFF, 0xFF,
  /* RLE: 001 Pixels @ 107,112*/ 1, 0xFF, 0xFF,
  /* RLE: 126 Pixels @ 108,112*/ 126, 0xFF, 0xFF,
  /* RLE: 001 Pixels @ 106,113*/ 1, 0xDE, 0xF7,
  /* RLE: 126 Pixels @ 107,113*/ 126, 0xFF, 0xFF,
  /* RLE: 001 Pixels @ 105,114*/ 1, 0xFF, 0xFF,
  /* RLE: 1686 Pixels @ 106,114*/ 254, 0xFF, 0xFF, 254, 0xFF, 0xFF, 254, 0xFF, 0xFF, 254, 0xFF, 0xFF, 254, 0xFF, 0xFF, 254, 0xFF, 0xFF, 162, 0xFF, 0xFF,


  0};  /* 8203 for 16384 pixels */

GUI_CONST_STORAGE GUI_BITMAP bmusb_image = {
  128, /* XSize */
  128, /* YSize */
  256, /* BytesPerLine */
  16, /* BitsPerPixel */
  (unsigned char *)acusb_image,  /* Pointer to picture data */
  NULL  /* Pointer to palette */
 ,GUI_DRAW_RLE16
};


static const char *aProgressString[] = {"/","-","\\","|"};

/*********************************************************************
*
*       Extern variables
*
**********************************************************************
*/
extern GUI_CONST_STORAGE GUI_BITMAP bmusb_image;


/*********************************************************************
*
*       Static ccode
*
**********************************************************************
*/

/*********************************************************************
*
*       _FSTest
*/
static void _FSTest(void) {
  FS_FILE    * pFile;
  unsigned     Len;
  const char * sInfo = "This sample based on the Segger emUSB software with MSD component.\r\nFor further information please visit: www.segger.com\r\n";

  Len = strlen(sInfo);
  if (FS_IsLLFormatted("") == 0) {
    FS_X_Log("Low level formatting");
    FS_FormatLow("");          /* Erase & Low-level  format the volume */
  }
  if (FS_IsHLFormatted("") == 0) {
    FS_X_Log("High level formatting\n");
    FS_Format("", NULL);       /* High-level format the volume */
  }
  pFile = FS_FOpen("Readme.txt", "w");
  FS_Write(pFile, sInfo, Len);
  FS_FClose(pFile);
  FS_Unmount("");
}

/*********************************************************************
*
*       _AddMSD
*
*  Function description
*    Add mass storage device to USB stack
*
*  Notes:
*   (1)  -   This examples uses the internal driver of the file system.
*            The module intializes the low-level part of the file system if necessary.
*            If FS_Init() was not previously called, none of the high level functions
*            such as FS_FOpen, FS_Write etc will work.
*            Only functions that are driver related will be called.
*            Initialization, sector read/write, retrieve device information.
*            The following members of the DriverData are used as follows:
*              DriverData.pStart       = Index no. of the volume that shall be used.
*              DriverData.NumSectors   = Number of sectors to be used - 0 means auto-detect.
*              DriverData.StartSector  = The first sector that shall be used.
*              DriverData.SectorSize will not be used.
*/
static void _AddMSD(void) {
  static U8 _abOutBuffer[USB_MAX_PACKET_SIZE];
  USB_MSD_INIT_DATA     InitData;
  USB_MSD_INST_DATA     InstData;

  InitData.EPIn  = USB_AddEP(1, USB_TRANSFER_TYPE_BULK, USB_MAX_PACKET_SIZE, NULL, 0);
  InitData.EPOut = USB_AddEP(0, USB_TRANSFER_TYPE_BULK, USB_MAX_PACKET_SIZE, _abOutBuffer, USB_MAX_PACKET_SIZE);
  USB_MSD_Add(&InitData);
  //
  // Add logical unit 0:
  //
  memset(&InstData, 0,  sizeof(InstData));
  InstData.pAPI                    = &USB_MSD_StorageByName;    // s. Note (1)
  InstData.DriverData.pStart       = "";
  USB_MSD_AddUnit(&InstData);
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       Get information that are used during enumeration
*/

/*********************************************************************
*
*       USB_GetVendorName
*
*  Function description
*    Returns vendor Id
*/
U16 USB_GetVendorId(void) {
  return 0x8765;
}

/*********************************************************************
*
*       USB_GetProductId
*
*  Function description
*    Returns the product Id
*/
U16 USB_GetProductId(void) {
  return 0x1000;
}

/*********************************************************************
*
*       USB_GetVendorName
*
*  Function description
*    Returns vendor name.
*/
const char * USB_GetVendorName(void) {
  return "Vendor";
}

/*********************************************************************
*
*       USB_GetProductName
*
*  Function description
*    Returns product name
*/
const char * USB_GetProductName(void) {
  return "MSD device";
}

/*********************************************************************
*
*       USB_GetSerialNumber
*
*  Function description
*    Returns serial number
*/
const char * USB_GetSerialNumber(void) {
  return "13245678";
}

/*********************************************************************
*
*       String information routines when inquiring the volume
*/
/*********************************************************************
*
*       USB_MSD_GetVendorName
*/
const char * USB_MSD_GetVendorName(U8 Lun) {
  return "Vendor";
}

/*********************************************************************
*
*       USB_MSD_GetProductName
*/
const char * USB_MSD_GetProductName(U8 Lun) {
  return "MSD Volume";
}

/*********************************************************************
*
*       USB_MSD_GetProductVer
*/
const char * USB_MSD_GetProductVer(U8 Lun) {
  return "1.00";
}

/*********************************************************************
*
*       USB_MSD_GetSerialNo
*/
const char * USB_MSD_GetSerialNo(U8 Lun) {
  return "134657890";
}

/*********************************************************************
*
*       MainTask
*
*/
void MainTask(void);
void MainTask(void) {
  char n = 0;
  GUI_Init();
  GUI_SetBkColor(GUI_WHITE);
  GUI_Clear();
  GUI_DrawBitmap(&bmusb_image,80,50);
  GUI_SetColor(GUI_BLACK);
  GUI_SetFont(&GUI_Font20B_ASCII);
  GUI_DispString("USB Mass Storage Device sample\n");
  USB_Init();
  FS_Init();
  _FSTest();
  _AddMSD();
  USB_Start();
  while (1) {
    //
    // Wait for configuration
    //
    while ((USB_GetState() & (USB_STAT_CONFIGURED | USB_STAT_SUSPENDED)) != USB_STAT_CONFIGURED) {
      BSP_ToggleLED(0);
      GUI_GotoXY(0,190);
      GUI_DispString("Enumeration...");
      GUI_DispString(aProgressString[n]);
      n++;
      n &= 0x03;
      USB_OS_Delay(50);
    }
    GUI_GotoXY(0,190);
    GUI_DispString("Connected!       ");
    BSP_SetLED(0);
    USB_MSD_Task();
  }
}

/**************************** end of file ***************************/

